# Generated by Django 5.2.6 on 2025-09-17 16:53

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("edc_adherence", "0004_alter_nonadherencereasons_options_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="nonadherencereasons",
            name="extra_value",
            field=models.CharField(default="", max_length=250),
        ),
        migrations.AlterField(
            model_name="nonadherencereasons",
            name="field_name",
            field=models.CharField(
                blank=True,
                default="",
                editable=False,
                help_text="Not required",
                max_length=25,
            ),
        ),
        migrations.AlterField(
            model_name="nonadherencereasons",
            name="plural_name",
            field=models.CharField(
                default="", max_length=250, verbose_name="Plural name"
            ),
        ),
    ]



from django.db import models

from ..model_mixin_factories import ogtt_model_mixin_factory


class OgttModelMixin(ogtt_model_mixin_factory("ogtt"), models.Model):
    """A model mixin of fields for the OGTT"""

    class Meta:
        abstract = True



from __future__ import annotations

from pathlib import Path
from typing import TYPE_CHECKING

from django.core.management.color import color_style

from edc_model_to_dataframe.model_to_dataframe import ModelToDataframe

from .csv_exporter import CsvExporter

if TYPE_CHECKING:
    from django.db.models import QuerySet

style = color_style()


class CsvModelExporter:
    df_maker_cls = ModelToDataframe
    csv_exporter_cls = CsvExporter

    def __init__(
        self,
        model: str | None = None,
        queryset: QuerySet | None = None,
        decrypt: bool | None = None,
        sort_by: list | tuple | str | None = None,
        export_folder: Path | None = None,
        site_ids: list[int] | None = None,
        **kwargs,
    ):
        self.model = model or queryset.model._meta.label_lower
        self.export_folder = export_folder
        self.df_maker = self.df_maker_cls(
            model=model,
            queryset=queryset,
            decrypt=decrypt,
            sites=site_ids or [],
            **kwargs,
        )
        self.csv_exporter = self.csv_exporter_cls(
            model_name=self.model,
            sort_by=sort_by,
            export_folder=export_folder,
            **kwargs,
        )

    def to_csv(self):
        dataframe = self.df_maker.dataframe
        return self.csv_exporter.to_csv(dataframe=dataframe, export_folder=self.export_folder)

    def to_stata(self):
        dataframe = self.df_maker.dataframe
        return self.csv_exporter.to_stata(
            dataframe=dataframe, export_folder=self.export_folder
        )



from __future__ import annotations

from django.db import models

from edc_constants.constants import NOT_APPLICABLE
from edc_lab.choices import GLUCOSE_UNITS_NA, RESULT_QUANTIFIER_NA

from ..constants import GLUCOSE_HIGH_READING


def glucose_model_mixin_factory(utest_id: str, **kwargs):
    class AbstractModel(models.Model):
        class Meta:
            abstract = True

    opts = {
        f"{utest_id}_value": models.DecimalField(
            verbose_name="Glucose level",
            max_digits=8,
            decimal_places=2,
            null=True,
            blank=True,
            help_text=f"A `HIGH` reading may be entered as {GLUCOSE_HIGH_READING}",
        ),
        f"{utest_id}_quantifier": models.CharField(
            verbose_name="Glucose quantifier",
            max_length=10,
            choices=RESULT_QUANTIFIER_NA,
            default=NOT_APPLICABLE,
        ),
        f"{utest_id}_units": models.CharField(
            verbose_name="Glucose units",
            max_length=15,
            choices=GLUCOSE_UNITS_NA,
            default=NOT_APPLICABLE,
        ),
        f"{utest_id}_date": models.DateField(
            verbose_name="Glucose date measured",
            null=True,
            blank=True,
        ),
    }

    opts.update(**kwargs)

    for name, fld_cls in opts.items():
        AbstractModel.add_to_class(name, fld_cls)

    return AbstractModel



from __future__ import annotations

from django.db import models

from ..model_mixin_factories import fasting_model_mixin_factory


class FastingModelMixin(fasting_model_mixin_factory(), models.Model):
    """A model mixin of fields about fasting.

    Used together with mixins for glucose measurements.
    """

    class Meta:
        abstract = True



from django.db.models.signals import post_save
from django.dispatch.dispatcher import receiver


@receiver(post_save, weak=False, dispatch_uid="update_timepoint_on_post_save")
def update_timepoint_on_post_save(sender, instance, raw, created, using, **kwargs):
    """Update the TimePointStatus mixin datetime field."""
    if not raw:
        try:
            instance.update_timepoint()
        except AttributeError as e:
            if "update_timepoint" not in str(e):
                raise
            pass



from django.conf import settings
from django.contrib import admin
from django.template.loader import render_to_string
from django.urls import reverse
from django.utils import timezone
from django.utils.html import format_html
from django.utils.safestring import mark_safe
from django_audit_fields.admin import audit_fieldset_tuple

from edc_appointment.models import Appointment
from edc_utils import convert_php_dateformat, formatted_age

from ...admin_site import edc_pharmacy_admin
from ...forms import RxRefillForm
from ...models import RxRefill
from ..model_admin_mixin import ModelAdminMixin


@admin.register(RxRefill, site=edc_pharmacy_admin)
class RxRefillAdmin(ModelAdminMixin, admin.ModelAdmin):
    show_object_tools = True

    ordering: tuple[str, ...] = ("refill_start_datetime",)

    autocomplete_fields = ("dosage_guideline", "formulation")

    form = RxRefillForm

    model = RxRefill

    fieldsets = (
        (
            None,
            {
                "fields": (
                    "rx",
                    "dosage_guideline",
                    "formulation",
                    "refill_start_datetime",
                    "refill_end_datetime",
                    "number_of_days",
                )
            },
        ),
        (
            "Optional Customizations",
            {
                "description": (
                    "This section is only required if customizing "
                    "the dosage guideline from above."
                ),
                "fields": ("weight_in_kgs",),
            },
        ),
        (
            "Notes",
            {"fields": ("notes",)},
        ),
        (
            "Verification",
            {"classes": ("collapse",), "fields": ("verified", "verified_datetime")},
        ),
        ("Calculations", {"classes": ("collapse",), "fields": ("total", "remaining")}),
        audit_fieldset_tuple,
    )

    list_display: tuple[str, ...] = (
        "subject_identifier",
        "dashboard",
        "duration",
        "description",
        "prescription",
        "active",
        "packed",
        "shipped",
        "received_at_site",
        "verified",
        "verified_datetime",
    )

    list_filter: tuple[str, ...] = (
        "active",
        "refill_start_datetime",
        "refill_end_datetime",
        "packed",
        "shipped",
        "received_at_site",
        "site",
    )

    search_fields: tuple[str, ...] = (
        "id",
        "site__id",
        "rx__id",
        "rx__rando_sid",
        "rx__subject_identifier",
        "rx__registered_subject__initials",
        "dosage_guideline__medication__name",
    )

    @admin.display(description="Subject identifier")
    def subject_identifier(self, obj=None):
        return obj.rx.subject_identifier

    @admin.display(description="Duration")
    def duration(self, obj=None):
        refill_start_date = obj.refill_start_datetime.strftime(
            convert_php_dateformat(settings.DATE_FORMAT)
        ).split(" ")
        refill_end_date = (
            obj.refill_end_datetime.strftime(
                convert_php_dateformat(settings.DATE_FORMAT)
            ).split(" ")
            if obj.refill_end_datetime
            else "???"
        )
        context = dict(
            refill_start_date=format_html(
                "{}",
                mark_safe("&nbsp;".join(refill_start_date)),  # nosec B703, B308
            ),
            refill_end_date=format_html(
                "{}",
                mark_safe("&nbsp;".join(refill_end_date)),  # nosec B703, B308
            ),
            number_of_days=obj.number_of_days,
        )
        return render_to_string("edc_pharmacy/duration.html", context=context)

    @admin.display(description="Rx")
    def prescription(self, obj=None):
        url = reverse("edc_pharmacy_admin:edc_pharmacy_rx_changelist")
        url = f"{url}?q={obj.rx.id}"
        context = dict(title="Back to RX", url=url, label="Rx")
        return render_to_string("edc_subject_dashboard/dashboard_button.html", context=context)

    @admin.display(description="Description of Refill")
    def description(self, obj=None):
        context = {
            "subject_identifier": obj.rx.registered_subject.subject_identifier,
            "initials": obj.rx.registered_subject.initials,
            "gender": obj.rx.registered_subject.gender,
            "age_in_years": formatted_age(
                born=obj.rx.registered_subject.dob, reference_dt=timezone.now()
            ),
            "number_of_days": obj.number_of_days,
            "remaining": obj.remaining,
            "total": obj.total,
            "SHORT_DATE_FORMAT": settings.SHORT_DATE_FORMAT,
            "rx_refill": obj,
        }
        return render_to_string(
            "edc_pharmacy/rx_refill_description.html",
            context,
        )

    def formfield_for_foreignkey(self, db_field, request, **kwargs):
        if db_field.name == "rx" and request.GET.get("rx"):
            kwargs["queryset"] = db_field.related_model.objects.filter(
                pk=request.GET.get("rx", 0)
            )
        return super().formfield_for_foreignkey(db_field, request, **kwargs)

    def get_subject_dashboard_url_kwargs(self, obj):
        appointment = Appointment.objects.filter(
            subject_identifier=obj.subject_identifier,
            appt_datetime__date__gte=obj.refill_start_datetime.date(),
        ).first()
        return dict(
            subject_identifier=obj.subject_identifier,
            appointment=appointment.id,
        )


class RxRefillInlineAdmin(admin.StackedInline):
    form = RxRefillForm

    model = RxRefill

    fields: tuple[str, ...] = (
        "dosage_guideline",
        "formulation",
        "refill_start_datetime",
        "refill_end_datetime",
        "number_of_days",
        "dose",
        "frequency",
        "frequency_units",
    )

    search_fields: tuple[str, ...] = "dosage_guideline__medication__name"

    ordering: tuple[str, ...] = ("refill_start_datetime",)

    extra = 0



# Generated by Django 3.2.13 on 2022-07-04 15:41

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("edc_export", "0011_auto_20220215_2308"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="historicaldatarequest",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical data request",
                "verbose_name_plural": "historical data requests",
            },
        ),
        migrations.AlterModelOptions(
            name="historicalexportreceipt",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical export receipt",
                "verbose_name_plural": "historical export receipts",
            },
        ),
        migrations.AlterModelOptions(
            name="historicalfilehistory",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical file history",
                "verbose_name_plural": "historical file historys",
            },
        ),
        migrations.AlterModelOptions(
            name="historicalobjecthistory",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical object history",
                "verbose_name_plural": "historical object historys",
            },
        ),
        migrations.AlterModelOptions(
            name="historicalplan",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical plan",
                "verbose_name_plural": "historical plans",
            },
        ),
        migrations.AlterField(
            model_name="historicaldatarequest",
            name="history_date",
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name="historicalexportreceipt",
            name="history_date",
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name="historicalfilehistory",
            name="history_date",
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name="historicalobjecthistory",
            name="history_date",
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name="historicalplan",
            name="history_date",
            field=models.DateTimeField(db_index=True),
        ),
    ]



from django.apps import apps as django_apps
from django.core.exceptions import ObjectDoesNotExist

from edc_constants.constants import UUID_PATTERN

from .fieldsets import Fieldsets


class FieldsetsModelAdminMixin:
    """A class that helps modify fieldsets for subject models

    * Model is expected to have a relation to have subject_visit__appointment.
    * Expects appointment to be in GET
    """

    appointment_model: str = "edc_appointment.appointment"
    # key: value where key is a visit_code. value is a fieldlist object
    conditional_fieldlists: dict = {}
    # key: value where key is a visit code. value is a fieldsets object.
    conditional_fieldsets: dict = {}

    fieldsets_move_to_end = None

    def get_key(self, request=None, **kwargs) -> str:
        """Returns a string that is the key to `get` the
        value in the "conditional" dictionaries.

        For example:
            appointment.visit_code == '1000' will be used
            to look into:
                conditional_fieldsets = {
                    '1000': ...}
        """
        visit_code = None
        appointment_model_cls = django_apps.get_model(self.appointment_model)
        appointment_id = request.GET.get("appointment")
        if appointment_id and UUID_PATTERN.match(appointment_id):
            try:
                appointment = appointment_model_cls.objects.get(pk=appointment_id)
            except ObjectDoesNotExist:
                pass
            else:
                visit_code = appointment.visit_code
                if appointment.visit_code_sequence != 0:
                    visit_code = f"{visit_code}.{appointment.visit_code_sequence}"
        return visit_code

    def get_fieldsets(self, request, obj=None):
        """Returns fieldsets after modifications declared in
        "conditional" dictionaries.
        """
        fieldsets = super().get_fieldsets(request, obj=obj)
        fieldsets = Fieldsets(fieldsets=fieldsets)
        key = self.get_key(request=request)
        fieldset = self.conditional_fieldsets.get(key)
        if fieldset:
            try:
                fieldset = tuple(fieldset)
            except TypeError:
                fieldset = (fieldset,)
            for f in fieldset:
                fieldsets.add_fieldset(fieldset=f)
        fieldlist = self.conditional_fieldlists.get(key)
        if fieldlist:
            try:
                fieldsets.insert_fields(
                    *fieldlist.insert_fields,
                    insert_after=fieldlist.insert_after,
                    section=fieldlist.section,
                )
            except AttributeError:
                pass
            try:
                fieldsets.remove_fields(*fieldlist.remove_fields, section=fieldlist.section)
            except AttributeError:
                pass
        fieldsets.move_to_end(self.fieldsets_move_to_end)
        return fieldsets.fieldsets



INVALID_SCREENING_IDENTIFIER = "INVALID_SCREENING_IDENTIFIER"


class ScreeningEligibilityError(Exception):
    pass


class ScreeningEligibilityAttributeError(Exception):
    pass


class ScreeningEligibilityModelAttributeError(Exception):
    pass


class ScreeningEligibilityCleanedDataKeyError(Exception):
    pass


class ScreeningEligibilityInvalidCombination(Exception):  # noqa: N818
    pass


class RequiredFieldValueMissing(Exception):  # noqa: N818
    pass


class InvalidScreeningIdentifierFormat(Exception):  # noqa: N818
    def __init__(self, *args, **kwargs):
        self.code = INVALID_SCREENING_IDENTIFIER
        super().__init__(*args, **kwargs)



# Generated by Django 4.2.5 on 2023-09-28 15:10
from django.db import migrations
from tqdm import tqdm


def update_new_site_field(apps, schema_editor):
    registered_subject_model_cls = apps.get_model("edc_registration.registeredsubject")
    model_cls = apps.get_model("edc_visit_schedule.subjectschedulehistory")
    qs = model_cls.objects.all()
    total = qs.count()
    for obj in tqdm(qs, total=total):
        obj.site_id = registered_subject_model_cls.objects.get(
            subject_identifier=obj.subject_identifier
        ).site_id
        obj.save_base(update_fields=["site_id"])


class Migration(migrations.Migration):
    dependencies = [
        ("edc_visit_schedule", "0012_alter_subjectschedulehistory_managers_and_more"),
    ]

    operations = [migrations.RunPython(update_new_site_field)]



from django.contrib import admin

from edc_model_admin.dashboard import ModelAdminSubjectDashboardMixin
from edc_model_admin.history import SimpleHistoryAdmin

from .admin_site import edc_offstudy_admin
from .models import SubjectOffstudy


@admin.register(SubjectOffstudy, site=edc_offstudy_admin)
class SubjectOffstudyAdmin(ModelAdminSubjectDashboardMixin, SimpleHistoryAdmin):
    fieldsets = (
        [
            None,
            {
                "fields": (
                    "subject_identifier",
                    "offstudy_datetime",
                )
            },
        ],
        [
            "Off-study reason",
            {
                "fields": (
                    "offstudy_reason",
                    "other_offstudy_reason",
                )
            },
        ],
        [
            "Comments",
            {"fields": ("comment",)},
        ],
    )

    list_filter = ("offstudy_datetime",)

    radio_fields = {
        "offstudy_reason": admin.VERTICAL,
    }



from __future__ import annotations

import re

from django.apps import apps as django_apps
from django.core.exceptions import ObjectDoesNotExist

from .exceptions import IdentifierError
from .models import IdentifierModel


class Identifier:
    name = "identifier"
    identifier_model_cls = IdentifierModel
    identifier_pattern = r"^\d+$"
    prefix_pattern: str | None = None
    prefix: str | None = None
    seed: str = "0"
    separator: str | None = None

    def __init__(self, last_identifier: str | None = None, prefix: str | None = None) -> None:
        self.identifier_as_list: list = []
        self.prefix: str = prefix or self.prefix or ""
        edc_device_app_config = django_apps.get_app_config("edc_device")
        self.device_id = edc_device_app_config.device_id
        self.identifier = (
            last_identifier or self.last_identifier or f"{self.prefix}{self.seed}"
        )
        self.prefix_pattern = f"^{self.prefix}$"
        self.identifier_pattern = self.prefix_pattern[:-1] + self.identifier_pattern[1:]
        if self.identifier:
            self.validate_identifier_pattern(self.identifier)
        self.next_identifier()

    def __repr__(self):
        return f"{self.__class__.__name__}('{self.identifier}')"

    def __str__(self):
        return self.identifier

    def __next__(self):
        self.next_identifier()
        return self.identifier

    def next(self):
        return self.__next__()

    def next_identifier(self):
        """Sets the next identifier and updates the identifier model."""
        identifier = self.remove_separator(self.identifier)
        identifier = self.increment(identifier)
        self.identifier = self.insert_separator(identifier)
        self.validate_identifier_pattern(self.identifier)
        self.update_identifier_model()

    @staticmethod
    def increment(identifier: int | str):
        return str(int(identifier or 0) + 1)

    def validate_identifier_pattern(self, identifier, pattern=None, error_msg=None):
        pattern = pattern or self.identifier_pattern
        try:
            identifier = re.match(pattern, identifier).group()
        except AttributeError as e:
            error_msg = error_msg or (
                "Invalid identifier format for pattern "
                f"{pattern}. Got identifier='{identifier}'"
            )
            raise IdentifierError(error_msg) from e
        return identifier

    @property
    def identifier_prefix(self):
        """Returns the prefix extracted from the identifier using
        the prefix_pattern.
        """
        if not self.prefix_pattern:
            return None
        return re.match(self.prefix_pattern[:-1], self.identifier).group()

    def update_identifier_model(self) -> bool:
        """Attempts to update identifier_model and returns True (or instance)
        if successful else False if identifier already exists.
        """
        try:
            self.identifier_model_cls.objects.get(identifier=self.identifier)
        except ObjectDoesNotExist:
            return self.identifier_model_cls.objects.create(
                identifier=self.identifier,
                identifier_type=self.name,
                identifier_prefix=self.identifier_prefix,
                device_id=self.device_id,
            )
        return False

    @property
    def last_identifier(self):
        """Returns the last identifier in the identifier model."""
        try:
            instance = (
                self.identifier_model_cls.objects.filter(identifier_type=self.name)
                .order_by("-sequence_number")
                .first()
            )
        except AttributeError:
            return None
        else:
            return instance.identifier

    def remove_separator(self, identifier):
        """Returns the identifier after removing the separator.

        The identifier is split into a list by separator and
        saved  as `identifier_as_list`.
        """
        if not identifier:
            return identifier
        self.identifier_as_list = identifier.split(self.separator)
        return "".join(self.identifier_as_list)

    def insert_separator(self, identifier):
        """Returns the identifier by reinserting the separator."""
        if not self.identifier_as_list:
            self.identifier_as_list = [identifier]
        start = 0
        items = []
        for item in self.identifier_as_list:
            items.append(identifier[start : start + len(item)])
            start += len(item)
        return (self.separator or "").join(items)



import pandas as pd

from .table import Table


class Visit(Table):
    default_columns = ["id", "subject_identifier", "report_datetime", "visit_code"]

    def prepare(self, df_subjects=None, **kwargs):
        self.dataframe = self.dataframe.rename(
            columns={"id": "subject_visit_id", "report_datetime": "visit_datetime"}
        )
        self.columns[self.columns.index("report_datetime")] = ["visit_datetime"]
        self.columns[self.columns.index("id")] = ["subject_visit_id"]
        self.dataframe["visit_datetime"] = self.helper.to_local_datetime(
            self.dataframe["visit_datetime"]
        )
        self.dataframe["visit_date"] = self.dataframe["visit_datetime"].dt.normalize()
        self.columns.append("visit_date")
        self.dataframe = pd.merge(
            self.dataframe, df_subjects, on="subject_identifier", how="left"
        )

    def read_csv(self, **kwargs):
        super().read_csv(**kwargs)
        self.dataframe["visit_datetime"] = self.dataframe["visit_datetime"].astype(
            "datetime64[ns]"
        )
        self.dataframe["visit_date"] = self.dataframe["visit_date"].astype("datetime64[ns]")
        self.dataframe["consent_datetime"] = self.dataframe["consent_datetime"].astype(
            "datetime64[ns]"
        )
        self.dataframe["consent_date"] = self.dataframe["consent_date"].astype(
            "datetime64[ns]"
        )



import os
from datetime import datetime

import pandas as pd

from ..helper import Helper as BaseHelper
from .table_to_dataframe import TableToDataframe


class Helper(BaseHelper):
    to_dataframe_cls = TableToDataframe

    def get_raw_dataframe(
        self,
        table_name=None,
        drop_sys_columns=None,
        csv_filename=None,
        path=None,
        timestamp=None,
    ):
        """Returns a dataframe."""
        if csv_filename:
            timestamp = timestamp or datetime.today().strftime("%Y%m%d%H%M%S")
            csv_filename = f"{table_name}_{timestamp}.csv"
            df_raw = self.to_dataframe_cls(
                table_name=table_name, drop_sys_columns=drop_sys_columns
            ).dataframe
            df_raw.to_csv(os.path.join(path, csv_filename), index=False)
        else:
            df_raw = pd.read_csv(os.path.join(path, csv_filename), low_memory=False)
        return df_raw



from edc_action_item.models import ActionNoManagersModelMixin
from edc_identifier.managers import SubjectIdentifierManager
from edc_model.models import BaseUuidModel, HistoricalRecords
from edc_sites.managers import CurrentSiteManager
from edc_sites.model_mixins import SiteModelMixin

from .constants import END_OF_STUDY_ACTION
from .model_mixins import OffstudyModelMixin


class SubjectOffstudy(
    OffstudyModelMixin,
    SiteModelMixin,
    ActionNoManagersModelMixin,
    BaseUuidModel,
):
    action_name = END_OF_STUDY_ACTION

    objects = SubjectIdentifierManager()

    on_site = CurrentSiteManager()

    history = HistoricalRecords()

    class Meta(BaseUuidModel.Meta):
        verbose_name = "Subject Offstudy"
        verbose_name_plural = "Subject Offstudy"
        indexes = (*ActionNoManagersModelMixin.Meta.indexes, *BaseUuidModel.Meta.indexes)



from __future__ import annotations

from dataclasses import dataclass, field
from urllib.parse import urlencode
from uuid import UUID

from django.urls import reverse

from edc_dashboard.url_names import url_names

__all__ = ["NextQuerystring"]


@dataclass(kw_only=True)
class NextQuerystring:
    """Make a querystring that complies with the next_url concept
    used by edc_submit_line and modeladminmixin on save/cancel to
    redirect to the dashboard.

    `extra_kwargs` may be added to prepopulate form fields.

    A querystring might look like this:
    ?next=namespace:urlname,attr1,attr2&attr1=value1&attr2=value2

    See DashboardModelButton for example usage.
    """

    next_url_name: str = field(default="subject_dashboard_url")
    reverse_kwargs: dict[str, str | int | float | UUID] = field(default_factory=dict)
    extra_kwargs: dict[str, str | int | float | UUID] = field(default_factory=dict)
    label: str = field(default="next")
    next_url: str = field(default=None, init=False)
    querystring: str = field(default=None, init=False)

    def __post_init__(self):
        # check valid "next" url name
        next_url_name = url_names.get_or_raise(self.next_url_name)
        # check "next" url reverses
        self.next_url = reverse(next_url_name, kwargs=self.reverse_kwargs)
        # build querystring
        keys = ",".join(list(self.reverse_kwargs.keys()))
        qs_part1 = f"{self.label}={next_url_name}" + "," + keys
        self.reverse_kwargs.update(self.extra_kwargs)
        qs_part2 = urlencode(self.reverse_kwargs)
        self.querystring = qs_part1 + "&" + qs_part2



from django.db import models

from edc_constants.choices import YES_NO
from edc_constants.constants import YES
from edc_crf.model_mixins import SingletonCrfModelMixin
from edc_dx import raise_on_unknown_diagnosis_labels
from edc_visit_schedule.utils import raise_if_baseline


class ClinicalReviewModelMixin(SingletonCrfModelMixin, models.Model):
    complications = models.CharField(
        verbose_name="Since last seen, has the patient had any complications",
        max_length=15,
        choices=YES_NO,
        help_text="If YES, complete the `Complications` CRF",
    )

    def save(self, *args, **kwargs):
        raise_if_baseline(self.related_visit)
        raise_on_unknown_diagnosis_labels(self, "_test", YES)
        super().save(*args, **kwargs)

    class Meta:
        abstract = True
        verbose_name = "Clinical Review"
        verbose_name_plural = "Clinical Review"



from django.conf import settings
from django.core.checks import Error


def edc_context_processors_check(
    app_configs, app_label=None, context_processor_name=None, error_code=None, **kwargs
):
    errors = []
    for template_config in settings.TEMPLATES:
        if context_processor_name not in template_config.get("OPTIONS").get(
            "context_processors"
        ):
            errors.append(  # noqa: PERF401
                Error(
                    "Missing item in TEMPLATE.OPTIONS.context_processors. "
                    f"Expected `{context_processor_name}`.",
                    id=f"{app_label}.{error_code or '001'}",
                )
            )
    return errors



from django.contrib import messages
from django.contrib.auth.mixins import LoginRequiredMixin
from django.http.response import HttpResponseRedirect
from django.urls.base import reverse
from django.views.generic.edit import ProcessFormView

from ..job_result import JobResult
from ..label import Label
from ..printers_mixin import PrintersMixin


class PrintLabelView(LoginRequiredMixin, PrintersMixin, ProcessFormView):
    success_url = "edc_label:home_url"
    label_cls = Label
    job_result_cls = JobResult

    def post(self, request, *args, **kwargs):
        printer_name = request.POST.get("printer_name")
        label_template_name = request.POST.get("label_template_name")
        printer = self.printers.get(printer_name)
        if printer.printer_state_message:
            messages.warning(request, f"{printer.name}: {printer.printer_state_message}")
        label = self.label_cls(label_template_name=label_template_name)
        zpl_data = label.render_as_zpl_data(copies=3, context={})
        job_id = printer.stream_print(zpl_data=zpl_data)
        job_result = self.job_result_cls(
            name=label_template_name, copies=1, job_ids=[job_id], printer=printer
        )
        messages.success(request, job_result.message)
        success_url = reverse(self.success_url)
        return HttpResponseRedirect(redirect_to=success_url)



from __future__ import annotations

import string
from typing import TYPE_CHECKING, Any

from django.core.exceptions import ValidationError
from django.db.models import Manager
from django.db.models.constants import LOOKUP_SEP
from django_crypto_fields.fields import BaseField as BaseEncryptedField

if TYPE_CHECKING:
    from edc_model.models import BaseUuidModel

    class MyModel(BaseUuidModel):
        class Meta(BaseUuidModel.Meta):
            pass


__all__ = ["ValueGetter", "ValueGetterInvalidLookup", "ValueGetterUnknownField"]


class ValueGetterUnknownField(ValidationError):
    pass


class ValueGetterInvalidLookup(ValidationError):
    pass


class ValueGetter:
    """A class to retrieve a field value from a django models
    instance.

    Keyword Args:
        * field_name: model instance field name
        * model_obj: model instance
        * lookups: a dictionary of {field_name: lookup, ...} where
            lookup is a django-style lookup string
            e.g. 'subject_visit__appointment__visit_code'.
        * decrypt: If False, replaces encrypted values with `encrypted_label`.
            (Default: False). If True will show the value.
        * additional_values: a class to provide additional attrs
            to get if field_name is not on the model_obj and
            not a lookup.
    """

    encrypted_label: str = "<encrypted>"
    m2m_delimiter: str = ";"

    def __init__(
        self,
        field_name: str = None,
        model_obj=None,
        lookups: dict | None = None,
        encrypt: bool | None = None,
        additional_values=None,
    ):
        self._value: Any = None
        self.additional_values = additional_values
        self.encrypt = encrypt
        self.field_name = field_name
        self.lookups = lookups or {}
        self.model_obj = model_obj
        self.model_cls = self.model_obj.__class__

    @property
    def value(self) -> Any:
        """Returns the "value"."""
        if not self._value:
            try:
                self._value = self._get_field_value(self.model_obj, self.field_name)
            except ValueGetterUnknownField as e:
                try:
                    self._value = getattr(self.model_obj, e.code)
                except AttributeError:
                    if self.additional_values:
                        self._value = getattr(self.additional_values, e.code)
                    else:
                        raise ValueGetterUnknownField(e)
            if self._value is None:
                self._value = ""
            self._value = self.strip_value(self._value)
        return self._value

    def _get_field_value(self, model_obj: type[MyModel] = None, field_name: str = None) -> Any:
        """Returns a field value.

        1. Tries to access a field as a model instance attribute;
        2. Via a lookup (e.g. get subject_identifier via subject_visit__subject_identifier)
        3. as an m2m
        """
        value = ""
        for f in model_obj.__class__._meta.get_fields():
            if (
                f.name == field_name
                and issubclass(f.__class__, BaseEncryptedField)
                and self.encrypt
            ):
                value = self.encrypted_label
        if value != self.encrypted_label:
            try:
                value = getattr(model_obj, field_name)
            except AttributeError:
                if field_name in self.lookups:
                    value = self.get_lookup_value(model_obj=model_obj, field_name=field_name)
                else:
                    raise ValueGetterUnknownField(
                        f"Unknown field name. Perhaps add a lookup. Got {field_name}.",
                        code=field_name,
                    )
            if isinstance(value, Manager):
                if field_name in self.m2m_field_names:
                    value = self.get_m2m_value(model_obj, field_name)
                else:
                    raise ValueGetterUnknownField(
                        f"Unknown field name. Got {field_name}.", code=field_name
                    )
        return value

    def get_lookup_value(self, model_obj: type[MyModel] = None, field_name: str = None) -> Any:
        """Returns the field value by following the lookup string
        to a related instance.
        """
        value = model_obj
        lookup_string = self.lookups.get(field_name)
        for attr in lookup_string.split(LOOKUP_SEP):
            try:
                value = getattr(value, attr)
            except AttributeError:
                raise ValueGetterInvalidLookup(
                    f"Invalid lookup string or value not set. Got {lookup_string}"
                )
        return value

    @property
    def m2m_field_names(self) -> list[str]:
        """Returns the list of m2m field names for this model."""
        return [m2m.name for m2m in self.model_cls._meta.many_to_many]

    def get_m2m_value(self, model_obj: type[MyModel] = None, field_name: str = None) -> str:
        """Returns an m2m field value as a delimited string."""
        return self.m2m_delimiter.join(
            [value.name for value in getattr(model_obj, field_name).all()]
        )

    def strip_value(self, value: Any) -> Any:
        """Returns a string cleaned of \n\t\r and double spaces."""
        try:
            value = value.replace(string.whitespace, " ")
        except (TypeError, AttributeError):
            pass
        else:
            value = " ".join(value.split())
        return value



# Generated by Django 5.1.2 on 2024-11-18 15:45

from django.db import migrations, models

import edc_pharmacy.models.stock.managers


class Migration(migrations.Migration):

    dependencies = [
        (
            "edc_pharmacy",
            "0042_remove_historicaldispensinghistory_history_user_and_more",
        ),
    ]

    operations = [
        migrations.CreateModel(
            name="StockProxy",
            fields=[],
            options={
                "proxy": True,
                "indexes": [],
                "constraints": [],
            },
            bases=("edc_pharmacy.stock",),
            managers=[
                ("objects", edc_pharmacy.models.stock.managers.StockManager()),
            ],
        ),
        migrations.AlterField(
            model_name="historicallot",
            name="lot_no",
            field=models.CharField(db_index=True, max_length=50, verbose_name="Batch"),
        ),
        migrations.AlterField(
            model_name="lot",
            name="lot_no",
            field=models.CharField(max_length=50, unique=True, verbose_name="Batch"),
        ),
    ]



# Generated by Django 2.0.7 on 2018-07-07 15:15

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [("edc_action_item", "0006_auto_20180707_1659")]

    operations = [
        migrations.RemoveField(model_name="actionitem", name="reference_identifier"),
        migrations.RemoveField(model_name="historicalactionitem", name="reference_identifier"),
    ]



from django.db import models

from .visit_code_fields_model_mixin import VisitCodeFieldsModelMixin
from .visit_schedule_fields_model_mixin import VisitScheduleFieldsModelMixin
from .visit_schedule_methods_model_mixin import VisitScheduleMethodsModelMixin


class VisitScheduleModelMixin(
    VisitScheduleFieldsModelMixin,
    VisitCodeFieldsModelMixin,
    VisitScheduleMethodsModelMixin,
    models.Model,
):
    """A model mixin for Appointment and related (subject) visit models.

    A model mixin that adds field attributes and methods that
    link a model instance to its schedule.

    This mixin is used with Appointment and Visit models via their
    respective model mixins.
    """

    class Meta:
        abstract = True



from django.db import models
from django.utils.html import format_html
from django.utils.safestring import mark_safe

from edc_constants.choices import YES_NO_NA
from edc_constants.constants import NOT_APPLICABLE
from edc_model import models as edc_models

from ...utils import get_list_model_app


class ClinicalReviewCholModelMixin(models.Model):
    chol_test = models.CharField(
        verbose_name="Since last seen, was the patient tested for high cholesterol?",
        max_length=15,
        choices=YES_NO_NA,
        default=NOT_APPLICABLE,
        help_text=format_html(
            "{}",
            mark_safe(
                "Note: Select `not applicable` if diagnosis previously reported. <BR>"
                "`Since last seen` includes today.<BR>"
                "If `yes', complete the initial review CRF<BR>"
                "If `not applicable`, complete the review CRF."
            ),  # nosec B308, B703
        ),
    )

    chol_test_date = models.DateField(
        verbose_name="Date test requested",
        null=True,
        blank=True,
    )

    chol_reason = models.ManyToManyField(
        f"{get_list_model_app()}.reasonsfortesting",
        related_name="chol_reason",
        verbose_name="Why was the patient tested for cholesterol?",
        blank=True,
    )

    chol_reason_other = edc_models.OtherCharField()

    chol_dx = models.CharField(
        verbose_name=format_html(
            "As of today, was the patient <u>{text}</u> diagnosed with high cholesterol?",
            text="newly",
        ),
        max_length=15,
        choices=YES_NO_NA,
        default=NOT_APPLICABLE,
    )

    class Meta:
        abstract = True



from edc_identifier.simple_identifier import SimpleUniqueIdentifier


class RequisitionIdentifier(SimpleUniqueIdentifier):
    random_string_length: int = 5
    identifier_type: str = "requisition_identifier"



# Generated by Django 2.2.2 on 2019-08-06 15:50

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [("edc_data_manager", "0003_auto_20190806_1749")]

    operations = [
        migrations.AlterUniqueTogether(
            name="dataquery",
            unique_together={("registered_subject", "rule_reference", "visit_schedule")},
        )
    ]



from __future__ import annotations

import collections
from typing import TYPE_CHECKING

from django.contrib import admin
from django.contrib.auth import get_permission_codename
from django.core.exceptions import FieldError, ObjectDoesNotExist
from django.db.models import QuerySet

from ..models import SiteProfile
from ..site import sites
from .list_filters import SiteListFilter

if TYPE_CHECKING:
    from django.contrib.admin import SimpleListFilter


__all__ = ["SiteModelAdminMixin"]


class SiteModeAdminMixinError(Exception):
    pass


class SiteModelAdminMixin:
    language_db_field_name = "language"

    limit_related_to_current_country: list[str] = None
    limit_related_to_current_site: list[str] = None
    site_list_display_insert_pos: int = 1

    def user_may_view_other_sites(self, request) -> bool:
        return sites.user_may_view_other_sites(request)

    def get_view_only_site_ids_for_user(self, request) -> list[int]:
        """Returns a list of sites, not including the current, that
        the user has permissions for.

        If the user has the model specific codename "viewallsites",
        returns all but the current (e.g. QA Reports model mixin).
        """
        if self.has_viewallsites_permission(request):
            return [
                s.id for s in request.user.userprofile.sites.all() if s.id != request.site.id
            ]
        return sites.get_view_only_site_ids_for_user(request=request)

    def has_viewallsites_permission(self, request, obj=None) -> bool:  # noqa: ARG002
        """Checks if the user has the EDC custom codename
        "viewallsites" for this model.

        See also: QA Reports model mixin.
        """
        opts = self.opts
        codename_allsites = get_permission_codename("viewallsites", opts)
        return request.user.has_perm(f"{opts.app_label}.{codename_allsites}")

    @admin.display(description="Site", ordering="site__id")
    def site_code(self, obj=None):
        return obj.site.id

    @admin.display(description="Site", ordering="site__id")
    def site_name(self, obj=None):
        try:
            site_profile = SiteProfile.objects.get(site__id=obj.site.id)
        except ObjectDoesNotExist:
            return obj.site.name
        return f"{site_profile.site.id} {site_profile.description}"

    def get_list_filter(self, request) -> tuple[str | type[SimpleListFilter], ...]:
        """Insert `SiteListFilter` before field name `created`.

        Remove site from the list if user does not have access
        to mulitple sites.
        """
        list_filter = super().get_list_filter(request)
        list_filter = [x for x in list_filter if x not in ("site", SiteListFilter)]
        if self.user_may_view_other_sites(request) or self.has_viewallsites_permission(
            request
        ):
            try:
                index = list_filter.index("created")
            except ValueError:
                index = len(list_filter)
            list_filter.insert(index, SiteListFilter)
        return tuple(list_filter)

    def get_list_display(self, request) -> tuple[str]:
        """Insert `site` after the first column"""
        list_display = super().get_list_display(request)
        pos = self.site_list_display_insert_pos
        if (
            self.user_may_view_other_sites(request)
            or self.has_viewallsites_permission(request)
        ) and "site" not in list_display:
            list_display = tuple(list_display)
            list_display = list_display[:pos], self.site_code, list_display[pos:]
        elif "site" in list_display:
            list_display = tuple(
                [x for x in list_display if x not in ["site", self.site_code]]
            )
            list_display = list_display[:pos], self.site_code, list_display[pos:]
        return list_display

    def get_queryset(self, request) -> QuerySet:
        """Limit modeladmin queryset for the current site only"""
        qs = super().get_queryset(request)
        site_ids = (request.site.id, *self.get_view_only_site_ids_for_user(request=request))
        try:
            qs = qs.select_related("site").filter(site_id__in=site_ids)
        except FieldError as e:
            raise SiteModeAdminMixinError(
                f"Model missing field `site`. Model `{self.model}`. Did you mean to use "
                f"the SiteModelAdminMixin? See `{self}`."
            ) from e
        return qs

    def get_form(self, request, obj=None, change=False, **kwargs):
        """Add current_site attr to form instance"""
        form = super().get_form(request, obj=obj, change=change, **kwargs)
        form.current_site = getattr(request, "site", None)
        form.current_locale = getattr(request, "LANGUAGE_CODE", None)
        return form

    def formfield_for_choice_field(self, db_field, request, **kwargs):
        """Use site id to select languages to show in choices."""
        if db_field.name == self.language_db_field_name:
            try:
                language_choices = sites.get_language_choices_tuple(request.site, other=True)
            except AttributeError as e:
                if "WSGIRequest" not in str(e):
                    raise
            else:
                if language_choices:
                    kwargs["choices"] = language_choices
        return super().formfield_for_choice_field(db_field, request, **kwargs)

    def formfield_for_foreignkey(self, db_field, request, **kwargs):
        """Filter a ForeignKey field`s queryset by the current site
        or country.

        Note, a queryset set by the ModelForm class will overwrite
        the field's queryset added here.
        """
        self.raise_on_dups_in_field_lists(
            self.limit_related_to_current_country,
            self.limit_related_to_current_site,
        )
        if db_field.name in (self.limit_related_to_current_country or []):
            self.raise_on_queryset_exists(db_field, kwargs)
            country = sites.get_current_country(request)
            model_cls = getattr(self.model, db_field.name).field.related_model
            kwargs["queryset"] = model_cls.objects.filter(siteprofile__country=country)
        elif db_field.name in (self.limit_related_to_current_site or []) and getattr(
            request, "site", None
        ):
            self.raise_on_queryset_exists(db_field, kwargs)
            model_cls = getattr(self.model, db_field.name).field.related_model
            kwargs["queryset"] = model_cls.objects.filter(id=request.site.id)
        elif db_field.name in (self.limit_related_to_current_site or []):
            self.raise_on_queryset_exists(db_field, kwargs)
            model_cls = getattr(self.model, db_field.name).field.related_model
            kwargs["queryset"] = model_cls.on_site.all()
        return super().formfield_for_foreignkey(db_field, request, **kwargs)

    def formfield_for_manytomany(self, db_field, request, **kwargs):
        """Filter a ManyToMany field`s queryset by the current site.

        Note, a queryset set by the ModelForm class will overwrite
        the field's queryset added here.
        """
        self.raise_on_dups_in_field_lists(
            self.limit_related_to_current_country,
            self.limit_related_to_current_site,
        )
        if db_field.name in (self.limit_related_to_current_site or []):
            self.raise_on_queryset_exists(db_field, kwargs)
            model_cls = getattr(self.model, db_field.name).remote_field.model
            kwargs["queryset"] = model_cls.on_site.all()
        elif db_field.name in (self.limit_related_to_current_country or []):
            country = sites.get_current_country(request)
            model_cls = getattr(self.model, db_field.name).remote_field.model
            kwargs["queryset"] = model_cls.objects.filter(siteprofile__country=country)
        return super().formfield_for_manytomany(db_field, request, **kwargs)

    def raise_on_queryset_exists(self, db_field, kwargs):
        """Raise an exception if the `queryset` key exists in the
        kwargs dict.

        If `queryset` exists, remove the field name from the class attr:
            limit_fk_field_to_...
            limit_m2m_field_to_...
        """
        if "queryset" in kwargs:
            raise SiteModeAdminMixinError(
                f"Key `queryset` unexpectedly exists. Got field `{db_field.name}` "
                f"from {self}."
                f". Did you manually set key `queryset` for field `{db_field.name}`?"
            )

    @staticmethod
    def raise_on_dups_in_field_lists(*field_lists: list[str]):
        orig = []
        for field_list in field_lists:
            orig.extend(field_list or [])
        if dups := [item for item, count in collections.Counter(orig).items() if count > 1]:
            raise SiteModeAdminMixinError(
                f"Related field appears in more than one list. Got {dups}."
            )



from django.contrib import admin

from ..admin_site import edc_export_admin
from ..models import ExportReceipt


@admin.register(ExportReceipt, site=edc_export_admin)
class ExportReceiptAdmin(admin.ModelAdmin):
    date_hierarchy = "received_datetime"
    list_display = ("export_uuid", "model", "timestamp", "received_datetime")
    list_filter = ("model", "received_datetime", "created", "modified")
    search_fields = ("export_uuid", "tx_pk", "id")



# Generated by Django 2.2.6 on 2019-11-01 21:33

from django.db import migrations

import edc_model_fields.fields.other_charfield


class Migration(migrations.Migration):
    dependencies = [("edc_offstudy", "0007_auto_20191024_1000")]

    operations = [
        migrations.AlterField(
            model_name="historicalsubjectoffstudy",
            name="offstudy_reason_other",
            field=edc_model_fields.fields.other_charfield.OtherCharField(
                blank=True,
                max_length=35,
                null=True,
                verbose_name="If other, please specify ...",
            ),
        ),
        migrations.AlterField(
            model_name="subjectoffstudy",
            name="offstudy_reason_other",
            field=edc_model_fields.fields.other_charfield.OtherCharField(
                blank=True,
                max_length=35,
                null=True,
                verbose_name="If other, please specify ...",
            ),
        ),
    ]



from edc_model_wrapper import ModelWrapper

from edc_lab.models import Aliquot, BoxItem, ManifestItem


class AliquotModelWrapper(ModelWrapper):
    model_cls = Aliquot
    next_url_name = "aliquot_listboard_url"

    @property
    def human_readable_identifier(self):
        return self.object.human_readable_identifier

    @property
    def box_item(self):
        try:
            return BoxItem.objects.get(identifier=self.aliquot_identifier)
        except BoxItem.DoesNotExist:
            return None

    @property
    def manifest_item(self):
        manifest_item = None
        if self.box_item:
            try:
                manifest_item = ManifestItem.objects.get(
                    identifier=self.box_item.box.box_identifier
                )
            except ManifestItem.DoesNotExist:
                pass
        return manifest_item



from edc_list_data.model_mixins import ListModelManager, ListModelMixin
from edc_model.models import HistoricalRecords


class ReasonsForTesting(ListModelMixin):
    objects = ListModelManager()
    history = HistoricalRecords()

    class Meta(ListModelMixin.Meta):
        verbose_name = "Reasons for Testing"
        verbose_name_plural = "Reasons for Testing"


class DiagnosisLocations(ListModelMixin):
    objects = ListModelManager()
    history = HistoricalRecords()

    class Meta(ListModelMixin.Meta):
        verbose_name = "Diagnosis Locations"
        verbose_name_plural = "Diagnosis Locations"



RANDOMIZED = "RANDOMIZED"

ACTIVE = "active"
ACTIVE_NAME = "Active: "
PLACEBO = "placebo"
PLACEBO_NAME = "Placebo: "
DEFAULT = "default"
DEFAULT_ASSIGNMENT_MAP: dict[str, int] = {ACTIVE: 1, PLACEBO: 2}
DEFAULT_ASSIGNMENT_DESCRIPTION_MAP: dict[str, str] = {
    ACTIVE: "Active",
    PLACEBO: "Placebo",
}
EXPORT_RANDO = "EXPORT_RANDO"
CONTROL = "control"
INTERVENTION = "intervention"



from django.db import models


class AeTmgMethodsModelMixin(models.Model):
    class Meta:
        abstract = True

    def __str__(self):
        return f"{self.action_identifier[-9:]}"

    def save(self, *args, **kwargs):
        self.subject_identifier = self.ae_initial.subject_identifier
        super().save(*args, **kwargs)

    def natural_key(self):
        return (self.action_identifier,)

    def get_action_item_reason(self):
        return self.ae_initial.ae_description

    def get_search_slug_fields(self) -> tuple[str]:
        fields = super().get_search_slug_fields()
        return *fields, "subject_identifier", "report_status"



# -*- coding: utf-8 -*-
# Generated by Django 1.10.4 on 2016-12-11 10:05
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [("edc_metadata", "0002_auto_20161127_2226")]

    operations = [
        migrations.AlterModelOptions(
            name="crfmetadata",
            options={
                "ordering": ("show_order",),
                "verbose_name": "Crf Metadata",
                "verbose_name_plural": "Crf Metadata",
            },
        ),
        migrations.AlterModelOptions(
            name="requisitionmetadata",
            options={
                "ordering": ("show_order",),
                "verbose_name": "Requisition Metadata",
                "verbose_name_plural": "Requisition Metadata",
            },
        ),
    ]



from edc_model_admin.admin_site import EdcAdminSite

from .apps import AppConfig

edc_pharmacy_admin = EdcAdminSite(
    name="edc_pharmacy_admin", app_label=AppConfig.name, keep_delete_action=True
)
edc_pharmacy_admin.disable_action("delete_selected")



from django.apps import apps as django_apps

from .model_exporter import ModelExporter


class PlanExporter:
    notification_cls = None
    model_exporter_cls = ModelExporter
    plan_model = "edc_export.plan"

    def __init__(self, plan_name=None):
        plan_model_cls = django_apps.get_model(self.plan_model)
        self.plan = plan_model_cls.objects.get(name=plan_name)

    def __str__(self):
        return self.plan.name

    def __repr__(self):
        return f"{self.__class__.__name__}({self.plan.name})"

    def export(self):
        model_exporter = self.model_exporter_cls(
            model=self.plan.model,
            field_names=self.plan.field_names,
            exclude_field_names=self.plan.exclude_field_names,
            lookups=self.plan.lookups,
            exclude_m2m=self.plan.exclude_m2m,
            encrypt=self.plan.encrypt,
        )
        path = model_exporter.export()
        if self.plan.notification_name:
            notification = self.notification_cls(
                name=self.plan.notification_name, plan=self.plan, path=path
            )
        notification.notify()
        return path



from edc_crf.crf_form_validator import CrfFormValidator

from ..form_validator_mixins import NextAppointmentCrfFormValidatorMixin


class NextAppointmentCrfFormValidator(NextAppointmentCrfFormValidatorMixin, CrfFormValidator):
    def clean(self):
        self.validate_date_is_on_clinic_day()
        super().clean()



# Generated by Django 2.2.2 on 2019-08-06 15:49

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [("edc_data_manager", "0002_auto_20190725_1822")]

    operations = [
        migrations.RemoveField(model_name="dataquery", name="tcc_user"),
        migrations.RemoveField(model_name="historicaldataquery", name="tcc_user"),
        migrations.AddField(
            model_name="dataquery",
            name="dm_user",
            field=models.ForeignKey(
                blank=True,
                help_text="select a name from the list",
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="dm_user",
                to="edc_data_manager.DataManagerUser",
                verbose_name="DM resolved by",
            ),
        ),
        migrations.AddField(
            model_name="historicaldataquery",
            name="dm_user",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                help_text="select a name from the list",
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="edc_data_manager.DataManagerUser",
                verbose_name="DM resolved by",
            ),
        ),
        migrations.AlterField(
            model_name="dataquery",
            name="resolved_datetime",
            field=models.DateTimeField(blank=True, null=True, verbose_name="DM resolved on"),
        ),
        migrations.AlterField(
            model_name="dataquery",
            name="site_response_status",
            field=models.CharField(
                choices=[
                    ("New", "New"),
                    ("open", "Open"),
                    ("feedback", "Feedback, awaiting data manager"),
                    ("resolved", "Resolved"),
                ],
                default="New",
                max_length=25,
                verbose_name="Site status",
            ),
        ),
        migrations.AlterField(
            model_name="dataquery",
            name="status",
            field=models.CharField(
                choices=[
                    ("open", "Open, awaiting site"),
                    ("closed", "Closed"),
                    ("closed_with_action", "Closed, with plan of action"),
                ],
                default="open",
                max_length=25,
                verbose_name="DM status",
            ),
        ),
        migrations.AlterField(
            model_name="dataquery",
            name="title",
            field=models.CharField(max_length=150, null=True),
        ),
        migrations.AlterField(
            model_name="historicaldataquery",
            name="resolved_datetime",
            field=models.DateTimeField(blank=True, null=True, verbose_name="DM resolved on"),
        ),
        migrations.AlterField(
            model_name="historicaldataquery",
            name="site_response_status",
            field=models.CharField(
                choices=[
                    ("New", "New"),
                    ("open", "Open"),
                    ("feedback", "Feedback, awaiting data manager"),
                    ("resolved", "Resolved"),
                ],
                default="New",
                max_length=25,
                verbose_name="Site status",
            ),
        ),
        migrations.AlterField(
            model_name="historicaldataquery",
            name="status",
            field=models.CharField(
                choices=[
                    ("open", "Open, awaiting site"),
                    ("closed", "Closed"),
                    ("closed_with_action", "Closed, with plan of action"),
                ],
                default="open",
                max_length=25,
                verbose_name="DM status",
            ),
        ),
        migrations.AlterField(
            model_name="historicaldataquery",
            name="title",
            field=models.CharField(max_length=150, null=True),
        ),
    ]



from __future__ import annotations

from edc_appointment.utils import update_appt_status_for_timepoint

from ..constants import KEYED, NOT_REQUIRED, REQUIRED
from ..utils import (
    get_crf_metadata,
    get_requisition_metadata,
    refresh_metadata_for_timepoint,
)


class MetadataViewError(Exception):
    pass


class MetadataViewMixin:
    panel_model: str = "edc_lab.panel"
    metadata_show_status: tuple[str] = (REQUIRED, KEYED)

    def get_context_data(self, **kwargs) -> dict:
        if self.appointment:
            # always refresh metadata / run rules
            refresh_metadata_for_timepoint(self.appointment, allow_create=True)
            referer = self.request.headers.get("Referer")
            if (
                referer
                and "subject_review_listboard" in referer
                and self.appointment.related_visit
            ):
                update_appt_status_for_timepoint(self.appointment.related_visit)
            crf_qs = self.get_crf_metadata()
            requisition_qs = self.get_requisition_metadata()
            kwargs.update(crfs=crf_qs, requisitions=requisition_qs)
        kwargs.update(
            NOT_REQUIRED=NOT_REQUIRED,
            REQUIRED=REQUIRED,
            KEYED=KEYED,
        )
        return super().get_context_data(**kwargs)

    def get_crf_metadata(self):
        return (
            get_crf_metadata(self.appointment)
            .filter(entry_status__in=self.metadata_show_status)
            .order_by("show_order")
        )

    def get_requisition_metadata(self):
        return (
            get_requisition_metadata(self.appointment)
            .filter(entry_status__in=self.metadata_show_status)
            .order_by("show_order")
        )



from __future__ import annotations

from typing import TYPE_CHECKING

from django.core.exceptions import ObjectDoesNotExist
from django.template.loader import render_to_string
from django.urls import NoReverseMatch, reverse
from django.utils.translation import gettext_lazy as _

from edc_dashboard.url_names import url_names
from edc_registration import get_registered_subject_model_cls

if TYPE_CHECKING:
    from edc_registration.models import RegisteredSubject


class ModelAdminDashboardMixin:
    subject_dashboard_url_name = "subject_dashboard_url"
    subject_listboard_url_name = "subject_listboard_url"
    screening_listboard_url_name = "screening_listboard_url"
    show_dashboard_in_list_display_pos = None
    view_on_site_label = _("Subject dashboard")

    def dashboard(self, obj=None, label=None) -> str:
        url = self.get_subject_dashboard_url(obj=obj)
        if not url:
            url = reverse(
                self.get_subject_dashboard_url_name(obj=obj),
                kwargs=self.get_subject_dashboard_url_kwargs(obj),
            )
        context = dict(title=_("Go to subject's dashboard"), url=url, label=label)
        return render_to_string("edc_subject_dashboard/dashboard_button.html", context=context)

    def get_screening_listboard_url_name(self) -> str:
        return url_names.get(self.screening_listboard_url_name)

    def get_subject_listboard_url_name(self) -> str:
        return url_names.get(self.subject_listboard_url_name)

    def get_subject_dashboard_url(self, obj=None) -> str | None:
        return None

    def get_subject_dashboard_url_name(self, obj=None) -> str:
        return url_names.get(self.subject_dashboard_url_name)

    def get_subject_dashboard_url_kwargs(self, obj) -> dict:
        return dict(subject_identifier=obj.subject_identifier)

    def get_post_url_on_delete_name(self, *args) -> str:
        return self.get_subject_dashboard_url_name()

    def post_url_on_delete_kwargs(self, request, obj) -> dict:
        return self.get_subject_dashboard_url_kwargs(obj)

    def get_registered_subject(self, obj) -> RegisteredSubject:
        return get_registered_subject_model_cls().objects.get(
            subject_identifier=obj.subject_identifier
        )

    def get_list_display(self, request) -> tuple[str, ...]:
        list_display = super().get_list_display(request)
        if (
            self.show_dashboard_in_list_display_pos is not None
            and self.dashboard not in list_display
        ):
            list_display = list(list_display)
            list_display.insert(self.show_dashboard_in_list_display_pos, self.dashboard)
            list_display = tuple(list_display)
        return list_display

    def view_on_site(self, obj) -> str:
        try:
            self.get_registered_subject(obj)
        except ObjectDoesNotExist:
            url = reverse(self.get_screening_listboard_url_name())
            if screening_identifier := getattr(obj, "screening_identifier", None):
                url = f"{url}?q={screening_identifier}"
        else:
            try:
                url = reverse(
                    self.get_subject_dashboard_url_name(),
                    kwargs=self.get_subject_dashboard_url_kwargs(obj),
                )
            except NoReverseMatch as e:
                if callable(super().view_on_site):
                    url = super().view_on_site(obj)
                else:
                    raise NoReverseMatch(f"{e}. See subject_dashboard_url_name for {self!r}.")
        return url



import pandas as pd

from ..constants import numeric_datatypes


def convert_numerics_from_model(source_df: pd.DataFrame, model_cls) -> pd.DataFrame:
    numeric_cols = []
    for field_cls in model_cls._meta.get_fields():
        if field_cls.get_internal_type() in numeric_datatypes:
            numeric_cols.append(field_cls.name)
    if numeric_cols:
        source_df[numeric_cols] = source_df[numeric_cols].apply(pd.to_numeric, errors="coerce")
    return source_df



from __future__ import annotations

from dataclasses import dataclass, field
from typing import TYPE_CHECKING, TypeVar

from django.utils.translation import gettext as _

from edc_appointment.constants import (
    CANCELLED_APPT,
    COMPLETE_APPT,
    IN_PROGRESS_APPT,
    INCOMPLETE_APPT,
    SKIPPED_APPT,
)
from edc_constants.constants import INCOMPLETE
from edc_view_utils import ADD, CHANGE, VIEW, DashboardModelButton

if TYPE_CHECKING:
    from django.contrib.sites.models import Site

    from edc_appointment.models import Appointment
    from edc_visit_tracking.model_mixins import VisitModelMixin

    RelatedVisitModel = TypeVar("RelatedVisitModel", bound=VisitModelMixin)

__all__ = ["RelatedVisitButton"]


@dataclass
class RelatedVisitButton(DashboardModelButton):
    appointment: Appointment = None
    model_obj: RelatedVisitModel = None
    labels: tuple[str, str, str] = field(default=("Start", "Visit Report", "Visit Report"))
    model_cls: type[RelatedVisitModel] = field(default=None)

    def __post_init__(self):
        pass

    @property
    def site(self) -> Site | None:
        """If model_obj is None, then Site should come from the
        CRFMetadata or RequisitionMetadata models.
        """
        return getattr(self.model_obj, "site", self.appointment.site)

    def color(self) -> str:
        """Shows as orange to direct user to edit the related visit
        following setting the appointment to in IN_PROGRESS_APPT.

        Note:
            In the background, setting an appointment to
            IN_PROGRESS_APPT updates the related visit
            document_status to INCOMPLETE. See also
            DocumentStatusModelMixin and AppointmentReasonUpdater.
        """
        color = self.colors[VIEW]
        if self.appointment.appt_status == IN_PROGRESS_APPT:
            if (
                self.model_obj and self.model_obj.document_status == INCOMPLETE
            ) or not self.model_obj:
                color = self.colors[ADD]
            else:
                color = self.colors[VIEW]  # default / grey
        elif self.appointment.appt_status in [COMPLETE_APPT, SKIPPED_APPT]:
            color = self.colors[CHANGE]
        return color

    @property
    def disabled(self) -> str:
        """Is only enabled if the related appointment is set to
        IN_PROGRESS_APPT.
        """
        disabled = "disabled"
        if self.appointment.appt_status == IN_PROGRESS_APPT and any(
            [self.perms.add, self.perms.change, self.perms.view]
        ):
            disabled = ""
        return disabled

    @property
    def label(self) -> str:
        label = _(super().label)
        if self.appointment.appt_status == SKIPPED_APPT:
            label = _("Skipped")
        elif self.appointment.appt_status == CANCELLED_APPT:
            label = _("Cancelled")
        elif self.appointment.appt_status == COMPLETE_APPT:
            label = _("Done")
        elif self.appointment.appt_status == INCOMPLETE_APPT:
            label = _("Incomplete")
        elif self.appointment.appt_status == IN_PROGRESS_APPT:
            label = _("Visit Report")
        return label

    @property
    def title(self):
        title = super().title
        if self.model_obj and self.model_obj.document_status == INCOMPLETE:
            title = _("Click to review before continuing to forms.")
        return title



from django.apps import apps as django_apps

from edc_auth.constants import PII, PII_VIEW
from edc_auth.site_auths import site_auths
from edc_export.constants import EXPORT


def update_site_auths() -> None:
    if django_apps.is_installed("edc_export"):
        site_auths.update_group("edc_registration.export_registeredsubject", name=EXPORT)
    site_auths.update_group(
        "edc_registration.display_dob",
        "edc_registration.display_firstname",
        "edc_registration.display_identity",
        "edc_registration.display_initials",
        "edc_registration.display_lastname",
        "edc_registration.view_historicalregisteredsubject",
        "edc_registration.view_registeredsubject",
        name=PII,
    )

    site_auths.update_group(
        "edc_registration.display_dob",
        "edc_registration.display_firstname",
        "edc_registration.display_identity",
        "edc_registration.display_initials",
        "edc_registration.display_lastname",
        "edc_registration.view_historicalregisteredsubject",
        "edc_registration.view_registeredsubject",
        name=PII_VIEW,
    )
    site_auths.add_pii_model("edc_registration.registeredsubject")


update_site_auths()



from edc_dashboard.auth_objects import dashboard_tuples

# group names
DATA_MANAGER = "DATA_MANAGER"
DATA_MANAGER_SUPER = "DATA_MANAGER_SUPER"
DATA_MANAGER_EXPORT = "DATA_MANAGER_EXPORT"
DATA_QUERY = "DATA_QUERY"
DATA_QUERY_VIEW = "DATA_QUERY_VIEW"
SITE_DATA_MANAGER = "SITE_DATA_MANAGER"

# role names
SITE_DATA_MANAGER_ROLE = "site_data_manager"
DATA_MANAGER_ROLE = "data_manager"

# codenames
data_manager = [
    "edc_data_manager.add_issue",
    "edc_data_manager.change_issue",
    "edc_data_manager.delete_issue",
    "edc_data_manager.view_issue",
    "edc_data_manager.add_dataquery",
    "edc_data_manager.add_queryrule",
    "edc_data_manager.change_dataquery",
    "edc_data_manager.change_queryrule",
    "edc_data_manager.delete_dataquery",
    "edc_data_manager.delete_queryrule",
    "edc_data_manager.view_crfdatadictionary",
    "edc_data_manager.view_datadictionary",
    "edc_data_manager.view_datamanageractionitem",
    "edc_data_manager.view_datamanageruser",
    "edc_data_manager.view_dataquery",
    "edc_data_manager.view_historicaldatadictionary",
    "edc_data_manager.view_historicaldataquery",
    "edc_data_manager.view_historicalqueryrule",
    "edc_data_manager.view_queryrule",
    "edc_data_manager.view_querysubject",
    "edc_data_manager.view_queryuser",
    "edc_data_manager.view_queryvisitschedule",
    "edc_data_manager.view_requisitiondatadictionary",
    "edc_data_manager.view_requisitionpanel",
    "edc_data_manager.view_visitdatadictionary",
    "edc_metadata.view_crfmetadata",
    "edc_metadata.view_requisitionmetadata",
]

custom_codenames = [
    "edc_data_manager.nav_data_manager_section",
]

data_manager.extend(custom_codenames)

data_manager.extend([tpl[0] for tpl in dashboard_tuples])

data_query = [c for c in data_manager if ("view_" in c or "navbar" in c)]

custom_codename_tuples = []
for codename in custom_codenames:
    custom_codename_tuples.append((codename, f"Can access {codename.split('.')[1]}"))



from __future__ import annotations

from datetime import datetime

from django import forms
from django.core.exceptions import ObjectDoesNotExist
from django.utils.text import format_lazy
from django.utils.translation import gettext_lazy as _

from edc_consent import site_consents
from edc_consent.consent_definition import ConsentDefinition
from edc_crf.crf_form_validator_mixins import BaseFormValidatorMixin


class PrnFormValidatorMixin(BaseFormValidatorMixin):
    """to be declared with PRN FormValidators."""

    report_datetime_field_attr = "report_datetime"

    @property
    def subject_consent(self):
        return self.get_consent_definition(
            report_datetime=self.report_datetime
        ).model_cls.objects.get(subject_identifier=self.subject_identifier)

    def get_consent_definition(
        self,
        report_datetime: datetime = None,
        fldname: str = None,
        error_code: str = None,
    ) -> ConsentDefinition:
        return site_consents.get_consent_definition(report_datetime=self.report_datetime)

    @property
    def report_datetime(self) -> datetime:
        """Returns report_datetime or raises.

        Report datetime is always a required field on a CRF model,
        Django will raise a field ValidationError before getting
        here if report_datetime is None.
        """
        report_datetime = None
        if self.report_datetime_field_attr in self.cleaned_data:
            report_datetime = self.cleaned_data.get(self.report_datetime_field_attr)
        elif self.instance:
            report_datetime = self.instance.report_datetime
        return report_datetime


class PrnSingletonModelFormMixin:
    def clean(self) -> dict:
        cleaned_data = super().clean()
        self.raise_if_singleton_exists()
        return cleaned_data

    def raise_if_singleton_exists(self) -> None:
        """Raise if singleton model instance exists."""
        if not self.instance.id:
            opts = {"subject_identifier": (self.get_subject_identifier())}
            try:
                self._meta.model.objects.get(**opts)
            except ObjectDoesNotExist:
                pass
            else:
                msg_string = _("Invalid. This form has already been submitted.")
                error_msg = format_lazy("{msg_string}", msg_string=msg_string)
                raise forms.ValidationError(error_msg)



# Generated by Django 5.2.6 on 2025-09-18 00:24

import django.db.models.deletion
import django.utils.timezone
import uuid
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("edc_data_manager", "0040_alter_datadictionary_device_created_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="dataquery",
            name="dm_user",
            field=models.ForeignKey(
                blank=True,
                help_text="Please select a name from the list",
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="edc_data_manager.datamanageruser",
                verbose_name="DM resolved by",
            ),
        ),
        migrations.AlterField(
            model_name="dataquery",
            name="locked_reason",
            field=models.TextField(
                blank=True,
                default="",
                help_text="If required, the reason the query cannot be resolved.",
                verbose_name="Reason query locked",
            ),
        ),
        migrations.AlterField(
            model_name="dataquery",
            name="plan_of_action",
            field=models.TextField(
                blank=True,
                default="",
                help_text="If required, provide a plan of action",
            ),
        ),
        migrations.AlterField(
            model_name="dataquery",
            name="report_datetime",
            field=models.DateTimeField(
                default=django.utils.timezone.now, verbose_name="Query date"
            ),
        ),
        migrations.AlterField(
            model_name="dataquery",
            name="rule_reference",
            field=models.CharField(
                default=uuid.uuid4, max_length=150, verbose_name="Query rule reference"
            ),
        ),
        migrations.AlterField(
            model_name="dataquery",
            name="sender",
            field=models.ForeignKey(
                help_text="Please select a name from the list",
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="edc_data_manager.datamanageruser",
                verbose_name="Query raised by",
            ),
        ),
        migrations.AlterField(
            model_name="dataquery",
            name="site_response_text",
            field=models.TextField(blank=True, default=""),
        ),
        migrations.AlterField(
            model_name="dataquery",
            name="subject_identifier",
            field=models.CharField(default="", editable=False, max_length=50),
        ),
        migrations.AlterField(
            model_name="dataquery",
            name="title",
            field=models.CharField(default="", max_length=150),
        ),
        migrations.AlterField(
            model_name="historicaldataquery",
            name="dm_user",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                help_text="Please select a name from the list",
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="edc_data_manager.datamanageruser",
                verbose_name="DM resolved by",
            ),
        ),
        migrations.AlterField(
            model_name="historicaldataquery",
            name="locked_reason",
            field=models.TextField(
                blank=True,
                default="",
                help_text="If required, the reason the query cannot be resolved.",
                verbose_name="Reason query locked",
            ),
        ),
        migrations.AlterField(
            model_name="historicaldataquery",
            name="plan_of_action",
            field=models.TextField(
                blank=True,
                default="",
                help_text="If required, provide a plan of action",
            ),
        ),
        migrations.AlterField(
            model_name="historicaldataquery",
            name="report_datetime",
            field=models.DateTimeField(
                default=django.utils.timezone.now, verbose_name="Query date"
            ),
        ),
        migrations.AlterField(
            model_name="historicaldataquery",
            name="rule_reference",
            field=models.CharField(
                default=uuid.uuid4, max_length=150, verbose_name="Query rule reference"
            ),
        ),
        migrations.AlterField(
            model_name="historicaldataquery",
            name="sender",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                help_text="Please select a name from the list",
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="edc_data_manager.datamanageruser",
                verbose_name="Query raised by",
            ),
        ),
        migrations.AlterField(
            model_name="historicaldataquery",
            name="site_response_text",
            field=models.TextField(blank=True, default=""),
        ),
        migrations.AlterField(
            model_name="historicaldataquery",
            name="subject_identifier",
            field=models.CharField(default="", editable=False, max_length=50),
        ),
        migrations.AlterField(
            model_name="historicaldataquery",
            name="title",
            field=models.CharField(default="", max_length=150),
        ),
    ]



import re
from copy import copy

from django.utils.html import format_html
from django.utils.safestring import mark_safe


class ModelAdminFormAutoNumberMixin:
    """Overrides get_form to insert question numbers and the DB
    field names.

    This is a mixin for `admin.ModelAdmin`

    Disable on the form by setting `form._meta.auto_number` to False.

    By default, auto_number it True.
    """

    skip_auto_numbering = []  # a list of fieldnames

    def auto_number(self, form):
        """Returns the form instance after inserting into the label
        question numbers and DB field names.
        """
        widget = 1
        start = getattr(form, "AUTO_NUMBER_START", 1)
        base_fields = {
            k: v for k, v in form.base_fields.items() if k not in self.skip_auto_numbering
        }
        for index, fld in enumerate(base_fields.items(), start=start):
            label = str(fld[widget].label)
            if not re.match(r"^\d+\.", label) and not re.match(r"\<a\ title\=\"", label):
                fld[widget].original_label = copy(label)
                fld[widget].label = format_html(
                    '<a title="{}">{}</a>. {}',
                    fld[0],
                    str(index),
                    mark_safe(label),  # nosec B308 B703
                )

        return form

    def get_form(self, request, obj=None, **kwargs):
        form = super().get_form(request, obj=obj, **kwargs)
        form = self.auto_number(form)
        return form



from django.contrib import admin
from django.template.loader import render_to_string
from django.urls import NoReverseMatch, reverse
from django.utils.translation import gettext as _
from django_audit_fields import audit_fields, audit_fieldset_tuple

from edc_dashboard.url_names import url_names
from edc_model_admin.dashboard import ModelAdminSubjectDashboardMixin
from edc_sites.admin import SiteModelAdminMixin

from ..admin_site import edc_visit_schedule_admin
from ..forms import SubjectScheduleHistoryForm
from ..models import SubjectScheduleHistory


@admin.register(SubjectScheduleHistory, site=edc_visit_schedule_admin)
class SubjectScheduleHistoryAdmin(
    SiteModelAdminMixin, ModelAdminSubjectDashboardMixin, admin.ModelAdmin
):
    form = SubjectScheduleHistoryForm

    date_hierarchy = "onschedule_datetime"

    fieldsets = (
        (
            None,
            {
                "fields": (
                    "subject_identifier",
                    "visit_schedule_name",
                    "schedule_name",
                    "schedule_status",
                    "onschedule_datetime",
                    "offschedule_datetime",
                    "onschedule_model",
                    "offschedule_model",
                )
            },
        ),
        audit_fieldset_tuple,
    )

    list_display = (
        "subject_identifier",
        "dashboard",
        "review",
        "visit_schedule_name",
        "schedule_name",
        "schedule_status",
        "onschedule_datetime",
        "offschedule_datetime",
    )

    search_fields = ("subject_identifier",)

    def get_list_filter(self, request) -> tuple:
        list_filter = super().get_list_filter(request)
        return (
            "schedule_status",
            "onschedule_datetime",
            "offschedule_datetime",
            "visit_schedule_name",
            "schedule_name",
            *list_filter,
        )

    def get_readonly_fields(self, request, obj=None) -> tuple:
        fields = super().get_readonly_fields(request, obj=obj)
        return (
            *fields,
            "subject_identifier",
            "visit_schedule_name",
            "schedule_name",
            "schedule_status",
            "onschedule_datetime",
            "offschedule_datetime",
            "onschedule_model",
            "offschedule_model",
            *audit_fields,
        )

    def dashboard(self, obj=None, label=None) -> str:
        try:
            url = reverse(
                self.get_subject_dashboard_url_name(),
                kwargs=self.get_subject_dashboard_url_kwargs(obj),
            )
        except NoReverseMatch:
            url = reverse(url_names.get("screening_listboard_url"), kwargs={})
            context = dict(
                title=_("Go to screening listboard"),
                url=f"{url}?q={obj.screening_identifier}",
                label=label,
            )
        else:
            context = dict(title=_("Go to subject dashboard"), url=url, label=label)
        return render_to_string("edc_subject_dashboard/dashboard_button.html", context=context)

    @staticmethod
    def review(obj=None) -> str:
        try:
            url = (
                f"{reverse('edc_review_dashboard:subject_review_listboard_url')}?"
                f"q={obj.subject_identifier}"
            )
        except NoReverseMatch:
            context = {}
        else:
            context = dict(title=_("Go to subject review dashboard"), url=url)
        return render_to_string(
            "edc_review_dashboard/subject_review_button.html", context=context
        )



from django.template.loader import render_to_string
from django.urls import reverse
from django.utils.html import format_html
from django.utils.safestring import mark_safe
from django.utils.translation import gettext_lazy as _


class ModelAdminChangelistButtonMixin:
    changelist_model_button_template_name = "edc_model_admin/changelist_model_button.html"

    def button(
        self,
        url_name,
        reverse_args,
        disabled=None,
        label=None,
        title=None,
        namespace=None,
    ):
        label = label or _("change")
        if namespace:
            url_name = f"{namespace}:{url_name}"
        url = reverse(url_name, args=reverse_args)
        return self.button_template(label=label, url=url, disabled=disabled, title=title)

    def change_button(
        self,
        url_name,
        reverse_args,
        disabled=None,
        label=None,
        title=None,
        namespace=None,
    ):
        label = label or _("change")
        if namespace:
            url_name = f"{namespace}:{url_name}"
        url = reverse(url_name, args=reverse_args)
        return self.button_template(label=label, url=url, disabled=disabled, title=title)

    def add_button(
        self,
        url_name,
        disabled=None,
        label=None,
        querystring=None,
        namespace=None,
        title=None,
    ):
        label = label or _("add")
        if namespace:
            url_name = f"{namespace}:{url_name}"
        url = reverse(url_name) + "" if querystring is None else querystring
        return self.button_template(label=label, url=url, disabled=disabled, title=title)

    def button_template(
        self,
        label: str = None,
        disabled: str | None = None,
        title: str | None = None,
        url: str | None = None,
    ):
        title = title or ""
        disabled = "disabled" if disabled else ""
        if disabled or not url:
            url = "#"
        context = dict(label=label, url=url, disabled=disabled, title=title)
        return format_html(
            "{html}",
            html=mark_safe(
                render_to_string(self.changelist_model_button_template_name, context)
            ),  # nosec B703, B308
        )



from django.conf import settings
from django.db import models
from django.utils.html import format_html
from django.utils.translation import gettext as _
from django_crypto_fields.fields import LastnameField
from django_crypto_fields.models import CryptoMixin

from edc_constants.choices import YES_NO

from ..validators import FullNameValidator, eligible_if_no


class VulnerabilityFieldsMixin(CryptoMixin, models.Model):
    is_incarcerated = models.CharField(
        verbose_name=_("Is the participant under involuntary incarceration?"),
        max_length=3,
        choices=YES_NO,
        validators=[eligible_if_no],
        null=True,
        help_text=_("If 'Yes' STOP participant cannot be consented."),
    )

    is_literate = models.CharField(
        verbose_name=_("Is the participant literate?"),
        max_length=3,
        choices=YES_NO,
        help_text=_(
            "If 'No' provide witness's name on this "
            "form and signature on the paper document."
        ),
    )

    witness_name = LastnameField(
        verbose_name=_("Witness's last and first name"),
        validators=[FullNameValidator()],
        blank=True,
        null=True,
        help_text=format_html(
            "{text1}.<BR>{text2}",
            text1="Required only if participant is illiterate",
            text2="Format is 'LASTNAME, FIRSTNAME'. All uppercase separated by a comma.",
        ),
    )

    language = models.CharField(
        verbose_name=_("Language of consent"),
        max_length=25,
        choices=settings.LANGUAGES,
        help_text=_(
            "The language used for the consent process will "
            "also be used during data collection."
        ),
    )

    class Meta:
        abstract = True



from django.db import models
from django.utils.html import format_html
from django_crypto_fields.fields import EncryptedCharField, EncryptedTextField

from edc_constants.choices import YES_NO
from edc_model.validators import cell_number, telephone_number


class SubjectWorkFieldsMixin(models.Model):
    may_call_work = models.CharField(
        max_length=25,
        choices=YES_NO,
        verbose_name=format_html(
            "{} <b>{}</b>, {} ",
            "Has the participant given permission to be contacted",
            "at work",
            "by telephone or cell, by study staff for follow-up purposes during the study?",
        ),
    )

    subject_work_place = EncryptedTextField(
        verbose_name="Name and location of work place",
        max_length=250,
        blank=True,
        null=True,
    )

    subject_work_phone = EncryptedCharField(
        verbose_name="Work contact telephone",
        validators=[telephone_number],
        blank=True,
        null=True,
    )

    subject_work_cell = EncryptedCharField(
        verbose_name="Work contact cell number",
        validators=[cell_number],
        blank=True,
        null=True,
    )

    class Meta:
        abstract = True



# Generated by Django 4.2.7 on 2023-12-04 22:19

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("edc_auth", "0030_alter_role_options_and_more"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="edcpermissions",
            options={
                "default_manager_name": "objects",
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "verbose_name": "Edc Permissions",
                "verbose_name_plural": "Edc Permissions",
            },
        ),
        migrations.AlterModelOptions(
            name="userprofile",
            options={
                "verbose_name": "User profile",
                "verbose_name_plural": "User profiles",
            },
        ),
        migrations.AddIndex(
            model_name="edcpermissions",
            index=models.Index(
                fields=["modified", "created"], name="edc_auth_ed_modifie_cd58ed_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="edcpermissions",
            index=models.Index(
                fields=["user_modified", "user_created"],
                name="edc_auth_ed_user_mo_817789_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="role",
            index=models.Index(
                fields=["modified", "created"], name="edc_auth_ro_modifie_fe8196_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="role",
            index=models.Index(
                fields=["user_modified", "user_created"],
                name="edc_auth_ro_user_mo_dc165e_idx",
            ),
        ),
    ]



# Generated by Django 5.2.3 on 2025-07-07 15:20
from django.core.exceptions import ObjectDoesNotExist
from django.db import migrations
from tqdm import tqdm

from edc_pharmacy.models import Confirmation, Stock


def update_confirm(apps, schema_editor):
    qs = Stock.objects.filter(confirmed=True)
    total = qs.count()
    for stock in tqdm(qs, total=total):
        try:
            Confirmation.objects.get(stock=stock)
        except ObjectDoesNotExist:
            obj = Confirmation(
                stock=stock,
                confirmed_by=stock.confirmed_by,
                confirmed_datetime=stock.confirmed_datetime,
            )
            obj.save()


class Migration(migrations.Migration):

    dependencies = [
        (
            "edc_pharmacy",
            "0082_alter_confirmation_managers_remove_confirmation_site_and_more",
        ),
    ]

    operations = [migrations.RunPython(update_confirm)]



from django.db import models
from django.utils import timezone

from edc_sites.model_mixins import SiteModelMixin

from ..panel_model_mixin import PanelModelMixin


class ResultModelMixin(PanelModelMixin, SiteModelMixin, models.Model):
    report_datetime = models.DateTimeField(null=True)

    pending_datetime = models.DateTimeField(default=timezone.now)

    pending = models.BooleanField(default=True)

    resulted_datetime = models.DateTimeField(default=timezone.now)

    resulted = models.BooleanField(default=False)

    class Meta:
        abstract = True



from django.utils.translation import gettext_lazy as _

__all__ = ["site_fieldset_tuple"]

site_fieldset_tuple: tuple[str, dict[str, tuple[str, ...]]] = (
    _("Site"),
    {"classes": ("collapse",), "fields": ["site"]},
)



from django.conf import settings

from edc_model.models import BaseUuidModel


class Listboard(BaseUuidModel):
    # see edc_auth for permissions attached to this model
    # create_edc_listboard_permissions

    pass


if settings.APP_NAME == "edc_listboard":
    from .tests import models  # noqa



import sys

from django.conf import settings

from .edc_permissions import EdcPermissions
from .role import Role
from .signals import (
    update_user_groups_on_role_m2m_changed,
    update_user_profile_on_post_save,
)
from .user_profile import UserProfile

if settings.APP_NAME == "edc_auth" and ("test" in sys.argv or "runtests.py" in sys.argv):
    from ..tests.models import *  # noqa



import os
import sys
from dataclasses import dataclass

from django.conf import settings
from django.core.checks import Error, Warning  # noqa: A004
from django.core.exceptions import ImproperlyConfigured
from django.core.management import color_style

from .blinding import get_unblinded_users, trial_is_blinded
from .site_randomizers import site_randomizers


@dataclass(frozen=True)
class Err:
    id: str
    cls: type[Warning | Error]


error_configs = dict(
    randomization_list_check=Err("edc_randomization.W001", Warning),
    blinded_trial_settings_check=Err("edc_randomization.E002", Error),
)

style = color_style()


def blinded_trial_settings_check(app_configs, **kwargs) -> list:
    errors = []
    error = error_configs.get("blinded_trial_settings_check")
    if not trial_is_blinded():
        try:
            get_unblinded_users()
        except ImproperlyConfigured:
            error_msg = (
                "Trial is not a blinded trial but users are listed as unblinded. "
                "See EDC_RANDOMIZATION_UNBLINDED_USERS"
            )
            errors.append(error.cls(error_msg, hint=None, obj=None, id=error.id))
    return errors


def randomizationlist_check(app_configs, **kwargs) -> list:
    sys.stdout.write(style.SQL_KEYWORD("randomizationlist_check ... \r"))
    errors = []
    error = error_configs.get("randomization_list_check")

    for randomizer in site_randomizers.registry.values():
        if kwargs.get("force_verify") or (
            "tox" not in sys.argv
            and "test" not in sys.argv
            and "runtests.py" not in sys.argv
            and "showmigrations" not in sys.argv
            and "makemigrations" not in sys.argv
            and "migrate" not in sys.argv
            and "shell" not in sys.argv
        ):
            error_msgs = randomizer.verify_list()
            for error_msg in error_msgs:
                errors.append(error.cls(error_msg, hint=None, obj=None, id=error.id))
        if not settings.DEBUG:
            if not randomizer.get_randomizationlist_path().is_relative_to(settings.ETC_DIR):
                errors.append(
                    Warning(
                        "Insecure configuration. Randomization list file must be "
                        "stored in the etc folder. See settings.ETC_DIR. Got "
                        f"{randomizer.get_randomizationlist_path()}",
                        hint="randomizationlist_path",
                        id="1000",
                    )
                )
            if os.access(str(randomizer.get_randomizationlist_path()), os.W_OK):
                errors.append(
                    Warning(
                        "Insecure configuration. File is writeable by this user. "
                        f"Got {randomizer.get_randomizationlist_path()}",
                        hint="randomizationlist_path",
                        id="1001",
                    )
                )
    sys.stdout.write(style.SQL_KEYWORD("randomization_list_check ... done.\n"))
    return errors



# Generated by Django 4.1 on 2022-08-21 18:56

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        (
            "edc_offstudy",
            "0012_rename_offstudy_reason_other_historicalsubjectoffstudy_other_offstudy_reason_and_more",
        ),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="historicalsubjectoffstudy",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical Subject Offstudy",
                "verbose_name_plural": "historical Subject Offstudy",
            },
        ),
        migrations.AlterModelOptions(
            name="subjectoffstudy",
            options={
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "get_latest_by": "modified",
                "ordering": ("-modified", "-created"),
                "verbose_name": "Subject Offstudy",
                "verbose_name_plural": "Subject Offstudy",
            },
        ),
    ]



# Generated by Django 5.2.6 on 2025-09-18 00:24

import django.utils.timezone
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("edc_action_item", "0039_alter_actionitem_auto_created_comment_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="actionitem",
            name="report_datetime",
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
        migrations.AlterField(
            model_name="historicalactionitem",
            name="report_datetime",
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
        migrations.AlterField(
            model_name="historicalreference",
            name="report_datetime",
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
        migrations.AlterField(
            model_name="reference",
            name="report_datetime",
            field=models.DateTimeField(default=django.utils.timezone.now),
        ),
    ]



from edc_auth.constants import (
    AUDITOR_ROLE,
    CLINICIAN_ROLE,
    CLINICIAN_SUPER_ROLE,
    NURSE_ROLE,
)
from edc_auth.site_auths import site_auths
from edc_auth.utils import remove_default_model_permissions_from_edc_permissions

SUBJECT_VIEW = "SUBJECT_VIEW"


def update_site_auths() -> None:
    site_auths.add_post_update_func(
        "edc_subject_dashboard", remove_default_model_permissions_from_edc_permissions
    )

    site_auths.add_custom_permissions_tuples(
        model="edc_subject_dashboard.edcpermissions",
        codename_tuples=(
            (
                "edc_subject_dashboard.view_subject_listboard",
                "Can access subject listboard",
            ),
            ("edc_subject_dashboard.nav_subject_section", "Can access nav_subject_section"),
        ),
    )

    site_auths.add_group(
        "edc_subject_dashboard.view_subject_listboard",
        "edc_subject_dashboard.nav_subject_section",
        name=SUBJECT_VIEW,
    )

    site_auths.update_role(SUBJECT_VIEW, name=CLINICIAN_ROLE)
    site_auths.update_role(SUBJECT_VIEW, name=NURSE_ROLE)
    site_auths.update_role(SUBJECT_VIEW, name=CLINICIAN_SUPER_ROLE)
    site_auths.update_role(SUBJECT_VIEW, name=AUDITOR_ROLE)


update_site_auths()



# Generated by Django 5.2.6 on 2025-09-17 16:53

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("edc_lab", "0035_alter_aliquot_revision_alter_box_revision_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="aliquot",
            name="comment",
            field=models.CharField(blank=True, default="", max_length=50),
        ),
        migrations.AlterField(
            model_name="aliquot",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="aliquot",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="aliquot",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="aliquot",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="aliquot",
            name="requisition_identifier",
            field=models.CharField(default="", editable=False, max_length=50),
        ),
        migrations.AlterField(
            model_name="aliquot",
            name="slug",
            field=models.CharField(
                db_index=True,
                default="",
                editable=False,
                help_text="a field used for quick search",
                max_length=250,
            ),
        ),
        migrations.AlterField(
            model_name="aliquot",
            name="subject_identifier",
            field=models.CharField(default="", editable=False, max_length=50),
        ),
        migrations.AlterField(
            model_name="box",
            name="comment",
            field=models.TextField(blank=True, default=""),
        ),
        migrations.AlterField(
            model_name="box",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="box",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="box",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="box",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="box",
            name="slug",
            field=models.CharField(
                db_index=True,
                default="",
                editable=False,
                help_text="a field used for quick search",
                max_length=250,
            ),
        ),
        migrations.AlterField(
            model_name="boxitem",
            name="comment",
            field=models.CharField(blank=True, default="", max_length=25),
        ),
        migrations.AlterField(
            model_name="boxitem",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="boxitem",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="boxitem",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="boxitem",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="boxitem",
            name="slug",
            field=models.CharField(
                db_index=True,
                default="",
                editable=False,
                help_text="a field used for quick search",
                max_length=250,
            ),
        ),
        migrations.AlterField(
            model_name="boxtype",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="boxtype",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="boxtype",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="boxtype",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="consignee",
            name="contact_name",
            field=models.CharField(blank=True, default="", max_length=50),
        ),
        migrations.AlterField(
            model_name="consignee",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="consignee",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="consignee",
            name="email",
            field=models.EmailField(blank=True, default="", max_length=50),
        ),
        migrations.AlterField(
            model_name="consignee",
            name="fax",
            field=models.CharField(blank=True, default="", max_length=50),
        ),
        migrations.AlterField(
            model_name="consignee",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="consignee",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="consignee",
            name="mobile",
            field=models.CharField(blank=True, default="", max_length=50),
        ),
        migrations.AlterField(
            model_name="consignee",
            name="state",
            field=models.CharField(
                blank=True, default="", max_length=50, verbose_name="State or Province"
            ),
        ),
        migrations.AlterField(
            model_name="consignee",
            name="telephone",
            field=models.CharField(blank=True, default="", max_length=50),
        ),
        migrations.AlterField(
            model_name="historicalaliquot",
            name="comment",
            field=models.CharField(blank=True, default="", max_length=50),
        ),
        migrations.AlterField(
            model_name="historicalaliquot",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="historicalaliquot",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="historicalaliquot",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="historicalaliquot",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="historicalaliquot",
            name="requisition_identifier",
            field=models.CharField(default="", editable=False, max_length=50),
        ),
        migrations.AlterField(
            model_name="historicalaliquot",
            name="slug",
            field=models.CharField(
                db_index=True,
                default="",
                editable=False,
                help_text="a field used for quick search",
                max_length=250,
            ),
        ),
        migrations.AlterField(
            model_name="historicalaliquot",
            name="subject_identifier",
            field=models.CharField(default="", editable=False, max_length=50),
        ),
        migrations.AlterField(
            model_name="historicalbox",
            name="comment",
            field=models.TextField(blank=True, default=""),
        ),
        migrations.AlterField(
            model_name="historicalbox",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="historicalbox",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="historicalbox",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbox",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbox",
            name="slug",
            field=models.CharField(
                db_index=True,
                default="",
                editable=False,
                help_text="a field used for quick search",
                max_length=250,
            ),
        ),
        migrations.AlterField(
            model_name="historicalboxitem",
            name="comment",
            field=models.CharField(blank=True, default="", max_length=25),
        ),
        migrations.AlterField(
            model_name="historicalboxitem",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="historicalboxitem",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="historicalboxitem",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="historicalboxitem",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="historicalboxitem",
            name="slug",
            field=models.CharField(
                db_index=True,
                default="",
                editable=False,
                help_text="a field used for quick search",
                max_length=250,
            ),
        ),
        migrations.AlterField(
            model_name="historicalconsignee",
            name="contact_name",
            field=models.CharField(blank=True, default="", max_length=50),
        ),
        migrations.AlterField(
            model_name="historicalconsignee",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="historicalconsignee",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="historicalconsignee",
            name="email",
            field=models.EmailField(blank=True, default="", max_length=50),
        ),
        migrations.AlterField(
            model_name="historicalconsignee",
            name="fax",
            field=models.CharField(blank=True, default="", max_length=50),
        ),
        migrations.AlterField(
            model_name="historicalconsignee",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="historicalconsignee",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="historicalconsignee",
            name="mobile",
            field=models.CharField(blank=True, default="", max_length=50),
        ),
        migrations.AlterField(
            model_name="historicalconsignee",
            name="state",
            field=models.CharField(
                blank=True, default="", max_length=50, verbose_name="State or Province"
            ),
        ),
        migrations.AlterField(
            model_name="historicalconsignee",
            name="telephone",
            field=models.CharField(blank=True, default="", max_length=50),
        ),
        migrations.AlterField(
            model_name="historicalmanifest",
            name="category_other",
            field=models.CharField(blank=True, default="", max_length=25),
        ),
        migrations.AlterField(
            model_name="historicalmanifest",
            name="comment",
            field=models.TextField(blank=True, default="", verbose_name="Comment"),
        ),
        migrations.AlterField(
            model_name="historicalmanifest",
            name="description",
            field=models.TextField(
                default="",
                help_text="If blank will be automatically generated",
                verbose_name="Description of contents",
            ),
        ),
        migrations.AlterField(
            model_name="historicalmanifest",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="historicalmanifest",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="historicalmanifest",
            name="export_references",
            field=models.TextField(blank=True, default=""),
        ),
        migrations.AlterField(
            model_name="historicalmanifest",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="historicalmanifest",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="historicalmanifest",
            name="slug",
            field=models.CharField(
                db_index=True,
                default="",
                editable=False,
                help_text="a field used for quick search",
                max_length=250,
            ),
        ),
        migrations.AlterField(
            model_name="historicalorder",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="historicalorder",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="historicalorder",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="historicalorder",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="historicalresult",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="historicalresult",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="historicalresult",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="historicalresult",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="historicalresultitem",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="historicalresultitem",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="historicalresultitem",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="historicalresultitem",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="historicalresultitem",
            name="quantifier",
            field=models.CharField(default="", max_length=25),
        ),
        migrations.AlterField(
            model_name="historicalresultitem",
            name="reference",
            field=models.CharField(default="", max_length=25),
        ),
        migrations.AlterField(
            model_name="historicalresultitem",
            name="utestid",
            field=models.CharField(default="", max_length=25),
        ),
        migrations.AlterField(
            model_name="historicalresultitem",
            name="value",
            field=models.CharField(default="", max_length=25),
        ),
        migrations.AlterField(
            model_name="historicalshipper",
            name="contact_name",
            field=models.CharField(blank=True, default="", max_length=50),
        ),
        migrations.AlterField(
            model_name="historicalshipper",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="historicalshipper",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="historicalshipper",
            name="email",
            field=models.EmailField(blank=True, default="", max_length=50),
        ),
        migrations.AlterField(
            model_name="historicalshipper",
            name="fax",
            field=models.CharField(blank=True, default="", max_length=50),
        ),
        migrations.AlterField(
            model_name="historicalshipper",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="historicalshipper",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="historicalshipper",
            name="mobile",
            field=models.CharField(blank=True, default="", max_length=50),
        ),
        migrations.AlterField(
            model_name="historicalshipper",
            name="state",
            field=models.CharField(
                blank=True, default="", max_length=50, verbose_name="State or Province"
            ),
        ),
        migrations.AlterField(
            model_name="historicalshipper",
            name="telephone",
            field=models.CharField(blank=True, default="", max_length=50),
        ),
        migrations.AlterField(
            model_name="manifest",
            name="category_other",
            field=models.CharField(blank=True, default="", max_length=25),
        ),
        migrations.AlterField(
            model_name="manifest",
            name="comment",
            field=models.TextField(blank=True, default="", verbose_name="Comment"),
        ),
        migrations.AlterField(
            model_name="manifest",
            name="description",
            field=models.TextField(
                default="",
                help_text="If blank will be automatically generated",
                verbose_name="Description of contents",
            ),
        ),
        migrations.AlterField(
            model_name="manifest",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="manifest",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="manifest",
            name="export_references",
            field=models.TextField(blank=True, default=""),
        ),
        migrations.AlterField(
            model_name="manifest",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="manifest",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="manifest",
            name="slug",
            field=models.CharField(
                db_index=True,
                default="",
                editable=False,
                help_text="a field used for quick search",
                max_length=250,
            ),
        ),
        migrations.AlterField(
            model_name="manifestitem",
            name="comment",
            field=models.CharField(blank=True, default="", max_length=25),
        ),
        migrations.AlterField(
            model_name="manifestitem",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="manifestitem",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="manifestitem",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="manifestitem",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="manifestitem",
            name="slug",
            field=models.CharField(
                db_index=True,
                default="",
                editable=False,
                help_text="a field used for quick search",
                max_length=250,
            ),
        ),
        migrations.AlterField(
            model_name="order",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="order",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="order",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="order",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="panel",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="panel",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="panel",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="panel",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="result",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="result",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="result",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="result",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="resultitem",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="resultitem",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="resultitem",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="resultitem",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="resultitem",
            name="quantifier",
            field=models.CharField(default="", max_length=25),
        ),
        migrations.AlterField(
            model_name="resultitem",
            name="reference",
            field=models.CharField(default="", max_length=25),
        ),
        migrations.AlterField(
            model_name="resultitem",
            name="utestid",
            field=models.CharField(default="", max_length=25),
        ),
        migrations.AlterField(
            model_name="resultitem",
            name="value",
            field=models.CharField(default="", max_length=25),
        ),
        migrations.AlterField(
            model_name="shipper",
            name="contact_name",
            field=models.CharField(blank=True, default="", max_length=50),
        ),
        migrations.AlterField(
            model_name="shipper",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="shipper",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="shipper",
            name="email",
            field=models.EmailField(blank=True, default="", max_length=50),
        ),
        migrations.AlterField(
            model_name="shipper",
            name="fax",
            field=models.CharField(blank=True, default="", max_length=50),
        ),
        migrations.AlterField(
            model_name="shipper",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="shipper",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="shipper",
            name="mobile",
            field=models.CharField(blank=True, default="", max_length=50),
        ),
        migrations.AlterField(
            model_name="shipper",
            name="state",
            field=models.CharField(
                blank=True, default="", max_length=50, verbose_name="State or Province"
            ),
        ),
        migrations.AlterField(
            model_name="shipper",
            name="telephone",
            field=models.CharField(blank=True, default="", max_length=50),
        ),
    ]



from edc_auth.site_auths import site_auths

from .auth_objects import (
    QA_REPORTS,
    QA_REPORTS_AUDIT,
    QA_REPORTS_AUDIT_ROLE,
    QA_REPORTS_ROLE,
    QA_REPORTS_SUPER_ROLE,
    qa_reports_codenames,
)


def update_site_auths() -> None:
    # groups
    site_auths.add_group(*qa_reports_codenames, name=QA_REPORTS)
    site_auths.add_group(*qa_reports_codenames, name=QA_REPORTS_AUDIT, view_only=True)

    # roles
    site_auths.add_role(QA_REPORTS, name=QA_REPORTS_ROLE)
    site_auths.add_role(QA_REPORTS, name=QA_REPORTS_SUPER_ROLE)
    site_auths.add_role(QA_REPORTS_AUDIT, name=QA_REPORTS_AUDIT_ROLE)


update_site_auths()



from __future__ import annotations

import sys
from copy import deepcopy
from typing import Any

from django.apps import apps as django_apps
from django.utils.module_loading import import_module, module_has_submodule


class AlreadyRegistered(Exception):
    pass


class SiteValuesMappingError(Exception):
    pass


class SiteValuesMappings:
    """
    # values_mappings.py
    import string

    from edc_pdutils.site_values_mappings import site_values_mappings

    from . import choices

    for attr in dir(choices):
        if attr[0] in string.ascii_uppercase:
            value = getattr(choices, attr)
            if isinstance(value, tuple):
                site_values_mappings.register(attr, getattr(choices, attr))

    """

    def __init__(self):
        self.registry = {}
        self.loaded = False

    def register(
        self,
        name: str = None,
        choices: tuple[tuple[str, Any]] = None,
        values_mapping: tuple[tuple[int, Any]] | None = None,
    ):
        if name in self.registry:
            raise AlreadyRegistered(f"Values mapping already registered. Got {name}.")
        if not values_mapping:
            values_mapping = self.generate_values_mapping(choices)
        self.registry.update({name: [choices, values_mapping]})
        self.loaded = True

    @staticmethod
    def generate_values_mapping(
        choices: tuple[tuple[str, Any]],
    ) -> tuple[tuple[int, str]]:
        values_mapping = []
        for index, choice in enumerate(choices):
            try:
                values_mapping.append((index, str(choice[1][::79])))
            except IndexError as e:
                raise IndexError(f"{e!s} Got {choice[1]}")
        return tuple(values_mapping)

    def get_by_choices(self, tpl: tuple[tuple[str, Any]]) -> tuple[tuple[int, str]] | None:
        for data in self.registry.values():
            if data[0] == tpl:
                return data[1]
        return None

    @staticmethod
    def autodiscover(module_name=None, verbose=True):
        """Autodiscovers values_mappings in the values_mappings.py file of
        any INSTALLED_APP.
        """
        before_import_registry = None
        module_name = module_name or "values_mappings"
        writer = sys.stdout.write if verbose else lambda x: x
        writer(f" * checking for site {module_name} (edc_pdutils) ...\n")
        for app in django_apps.app_configs:
            try:
                mod = import_module(app)
                try:
                    before_import_registry = deepcopy(site_values_mappings.registry)
                    import_module(f"{app}.{module_name}")
                    writer(f"   - registered values mapping '{module_name}' from '{app}'\n")
                except ImportError as e:
                    site_values_mappings.registry = before_import_registry
                    if module_has_submodule(mod, module_name):
                        raise SiteValuesMappingError(f"{e!s}. See {app}.{module_name}")
            except ImportError:
                pass


site_values_mappings = SiteValuesMappings()



from urllib.parse import unquote, urlencode

from django import template
from django.core.exceptions import ObjectDoesNotExist
from django.utils.safestring import mark_safe

from ..models import SubjectScheduleHistory

register = template.Library()


@register.inclusion_tag("edc_visit_schedule/subject_schedule_footer_row.html")
def subject_schedule_footer_row(
    subject_identifier, visit_schedule, schedule, subject_dashboard_url
):
    context = dict(
        visit_schedule=visit_schedule,
        verbose_name=schedule.offschedule_model_cls._meta.verbose_name,
        schedule=schedule,
    )

    try:
        history_obj = SubjectScheduleHistory.objects.get(
            visit_schedule_name=visit_schedule.name,
            schedule_name=schedule.name,
            subject_identifier=subject_identifier,
            # offschedule_datetime__isnull=False,
        )
    except ObjectDoesNotExist:
        history_obj = None
    context.update(history_obj=history_obj)
    if not history_obj:
        # subject was NEVER on this schedule
        context = dict(
            offschedule_datetime=None,
            onschedule_datetime=None,
            href=None,
        )
    elif not history_obj.offschedule_datetime:
        # subject is still ON this schedule
        onschedule_model_obj = schedule.onschedule_model_cls.objects.get(
            subject_identifier=subject_identifier
        )
        options = dict(subject_identifier=subject_identifier)
        query = unquote(urlencode(options))
        href = (
            f"{schedule.offschedule_model_cls().get_absolute_url()}?next="
            f"{subject_dashboard_url},subject_identifier"
        )
        href = "&".join([href, query])
        context.update(
            offschedule_datetime=None,
            onschedule_datetime=onschedule_model_obj.onschedule_datetime,
            href=mark_safe(href),  # nosec B703, B308  # noqa: S308
        )
    elif history_obj.offschedule_datetime:
        # subject is OFF this schedule (offschedule_model_obj)
        onschedule_model_obj = schedule.onschedule_model_cls.objects.get(
            subject_identifier=subject_identifier
        )
        offschedule_model_obj = schedule.offschedule_model_cls.objects.get(
            subject_identifier=subject_identifier
        )
        options = dict(subject_identifier=subject_identifier)
        query = unquote(urlencode(options))
        href = (
            f"{offschedule_model_obj.get_absolute_url()}?next="
            f"{subject_dashboard_url},subject_identifier"
        )
        href = "&".join([href, query])
        context.update(
            offschedule_datetime=history_obj.offschedule_datetime,
            onschedule_datetime=onschedule_model_obj.onschedule_datetime,
            href=mark_safe(href),  # nosec B703, B308  # noqa: S308
        )
    return context



# Generated by Django 5.0 on 2024-01-24 18:10

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        (
            "edc_export",
            "0018_remove_objecthistory_edc_export__modifie_36e5ad_idx_and_more",
        ),
        ("sites", "0002_alter_domain_unique"),
    ]

    operations = [
        migrations.AlterField(
            model_name="datarequest",
            name="site",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="datarequesthistory",
            name="site",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="historicaldatarequest",
            name="site",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="sites.site",
            ),
        ),
    ]



from django.urls.conf import path
from django.views.generic.base import RedirectView

from .admin_site import edc_registration_admin

app_name = "edc_registration"

urlpatterns = [
    path("admin/", edc_registration_admin.urls),
    path("", RedirectView.as_view(url="/edc_registration/admin/"), name="home_url"),
]



class NotConsentedError(Exception):
    pass


class ConsentVersionSequenceError(Exception):
    pass


class ConsentError(Exception):
    pass


class ConsentDefinitionError(Exception):
    pass


class ConsentDefinitionModelError(Exception):
    pass


class ConsentDefinitionDoesNotExist(Exception):
    pass


class ConsentDefinitionValidityPeriodError(Exception):
    pass


class AlreadyRegistered(Exception):
    pass


class SiteConsentError(Exception):
    pass


class ConsentDefinitionNotConfiguredForUpdate(Exception):
    pass


class ConsentExtensionDefinitionError(Exception):
    pass


class ConsentExtensionDefinitionModelError(Exception):
    pass



from django import forms
from django.apps import apps as django_apps

from edc_constants.constants import CANCELLED, NEW, OPEN
from edc_model_form.mixins import BaseModelFormMixin

from ..models import ActionItem


class ActionItemForm(BaseModelFormMixin, forms.ModelForm):
    subject_identifier = forms.CharField(
        label="Subject Identifier",
        required=False,
        widget=forms.TextInput(attrs={"readonly": "readonly"}),
    )

    def clean(self):
        cleaned_data = super().clean()
        self.force_open_status()
        self.raise_if_cannot_cancel()
        return cleaned_data

    def force_open_status(self) -> None:
        """Sets status to open for edited NEW action items."""
        if self.instance.id and self.cleaned_data.get("status") == NEW:
            self.cleaned_data["status"] = OPEN

    def raise_if_cannot_cancel(self) -> None:
        if (
            self.instance.id
            and self.cleaned_data.get("status") == CANCELLED
            and django_apps.get_model(self.instance.reference_model)
            .objects.filter(action_identifier=self.instance.action_identifier)
            .exists()
        ):
            raise forms.ValidationError({"status": "Invalid. This action cannot be cancelled"})

    class Meta:
        model = ActionItem
        fields = "__all__"



from .draw_bulk_stock_label_code39 import draw_bulk_stock_label_code39
from .draw_bulk_stock_label_code128 import draw_bulk_stock_label_code128
from .draw_label_watermark import draw_label_watermark
from .draw_patient_stock_label_code128 import draw_patient_stock_label_code128
from .draw_vertical_barcode_only_code128 import draw_vertical_barcode_only_code128
from .label_data import LabelData



from typing import Any

from django.db import models

from edc_constants.choices import DOCUMENT_STATUS
from edc_constants.constants import COMPLETE


class DocumentStatusModelMixin(models.Model):
    document_status = models.CharField(
        verbose_name="Document status",
        max_length=25,
        choices=DOCUMENT_STATUS,
        default=COMPLETE,
        help_text="If some data is still pending, flag as incomplete",
    )

    document_status_comments = models.TextField(
        verbose_name="Any comments related to status of this document",
        default="",
        blank=True,
        help_text="for example, why some data is still pending",
    )

    def save(self: Any, *args, **kwargs):
        self.update_document_status_on_save(kwargs.get("update_fields"))
        super().save(*args, **kwargs)

    def update_document_status_on_save(self, update_fields: list | None = None) -> None:
        """Updates `document_status` as complete unless field is listed
        in update_fields.

        Used when this instance (subject_visit) needs to be updated.
        For example, after being auto-created and before moving on to CRFs.

        See also: edc_subject_dashboard
        """
        if "document_status" not in (update_fields or []):
            self.document_status = COMPLETE

    class Meta:
        abstract = True



from __future__ import annotations

from dataclasses import dataclass, field
from typing import TYPE_CHECKING
from uuid import UUID

from django.apps import apps as django_apps
from django.utils.translation import gettext as _

from .dashboard_model_button import DashboardModelButton

if TYPE_CHECKING:
    from edc_registration.models import RegisteredSubject
    from edc_visit_schedule.models import VisitSchedule


__all__ = ["QueryButton"]


class QueryButtonError(Exception):
    pass


@dataclass
class QueryButton(DashboardModelButton):
    """Dashboard button for access to the DataQuery model
    from edc_data_manager.

    Note:
        visit_schedule: a model instance of VisitSchedule
        See edc_visit_schedule.
    """

    registered_subject: RegisteredSubject = None
    visit_schedule: VisitSchedule = None
    fa_icons: tuple[str, str, str] = field(default=(3 * ("",)))
    colors: tuple[str, str, str] = field(default=(3 * ("default",)))
    verbose_name: str = field(default=None, init=False)

    def __post_init__(self):
        if self.model_obj is not None:
            raise ValueError(f"Invalid. Expected none for 'model_obj'. Got {self.model_obj}.")
        self.model_cls = django_apps.get_model("edc_data_manager.dataquery")
        self.verbose_name = self.metadata_model_obj.verbose_name

    def label(self) -> str:
        return _("Query")

    @property
    def url(self) -> str:
        return "?".join([f"{self.model_cls().get_absolute_url()}", self.querystring])

    @property
    def extra_kwargs(self) -> dict[str, str | int | UUID]:
        if not self.registered_subject:
            self.registered_subject = django_apps.get_model(
                "edc_registration.registeredsubject"
            ).objects.get(subject_identifier=self.appointment.subject_identifier)
        if not self.visit_schedule:
            visit_schedule_model_cls = django_apps.get_model(
                "edc_visit_schedule.visitschedule"
            )
            self.visit_schedule = visit_schedule_model_cls.objects.get(
                visit_schedule_name=self.appointment.visit_schedule_name,
                schedule_name=self.appointment.schedule_name,
                visit_code=self.appointment.visit_code,
            )
        return dict(
            registered_subject=self.registered_subject.id,
            sender=self.user.id,
            visit_schedule=self.visit_schedule.id,
            visit_code_sequence=self.appointment.visit_code_sequence,
            title=self.verbose_name,
        )



from django.db import models

from edc_constants.choices import YES_NO
from edc_constants.constants import NOT_APPLICABLE

from ...choices import CHOL_MANAGEMENT
from ..dx_location_model_mixin import DxLocationModelMixin
from .ncd_initial_review_model_mixin import NcdInitialReviewModelMixin


class CholInitialReviewModelMixin(
    DxLocationModelMixin,
    NcdInitialReviewModelMixin,
):
    ncd_condition_label = "cholesterol"

    managed_by = models.CharField(
        verbose_name="How is the patient's cholesterol managed?",
        max_length=25,
        choices=CHOL_MANAGEMENT,
        default=NOT_APPLICABLE,
    )

    chol_performed = models.CharField(
        verbose_name=(
            "Has the patient had their cholesterol measured in the last few months?"
        ),
        max_length=15,
        choices=YES_NO,
    )

    class Meta:
        abstract = True
        verbose_name = "High Cholesterol Initial Review"
        verbose_name_plural = "High Cholesterol Initial Reviews"



import contextlib

from django.conf import settings
from django.contrib import admin
from django.contrib.admin.sites import AlreadyRegistered
from django.contrib.sites.models import Site
from django.http import HttpResponseRedirect
from django.urls import reverse
from django_audit_fields.admin import audit_fields, audit_fieldset_tuple

from edc_label import Label
from edc_model_admin.mixins import TemplatesModelAdminMixin
from edc_protocol.research_protocol_config import ResearchProtocolConfig

from .admin_site import edc_randomization_admin
from .auth_objects import RANDO_UNBLINDED
from .blinding import user_is_blinded
from .site_randomizers import site_randomizers

__all__ = ["RandomizationListModelAdmin", "print_pharmacy_labels"]


@admin.action(permissions=["view"], description="Print labels for pharmacy")
def print_pharmacy_labels(modeladmin, request, queryset):
    zpl_data = []
    label = Label(
        label_template_name="rando_pharmacy_label.lbl",
        static_files_path="edc_pharmacy/label_templates",
    )
    for obj in queryset:
        context = dict(
            protocol=ResearchProtocolConfig().protocol,
            protocol_title=ResearchProtocolConfig().protocol_title,
            site=obj.site_name.upper(),
            subject_identifier=obj.subject_identifier,
            barcode_value=obj.sid,
            sid=obj.sid,
        )
        keys = [k for k in context]
        for fld in obj._meta.get_fields():
            if fld.name not in keys and fld.name != "assignment":
                context.update({fld.name: getattr(obj, fld.name)})
        zpl_data.append(
            str(label.render_as_zpl_data(copies=1, context=context, encoding=False))
            .strip("\n")
            .replace("\n", "")
        )
    request.session["zpl_data"] = "|".join(zpl_data)
    url = reverse("edc_label:browser_print_labels_url")
    return HttpResponseRedirect(url)


class RandomizationListModelAdmin(TemplatesModelAdminMixin, admin.ModelAdmin):
    list_per_page = 15

    actions = (print_pharmacy_labels,)

    view_on_site = False

    ordering = ("sid",)

    search_fields = ("subject_identifier", "sid")

    def get_fieldsets(self, request, obj=None):  # noqa: ARG002
        return (
            (None, {"fields": self.get_fieldnames(request)}),
            audit_fieldset_tuple,
        )

    def get_readonly_fields(self, request, obj=None):
        readonly_fields = super().get_readonly_fields(request, obj=obj)
        readonly_fields = (
            *readonly_fields,
            "subject_identifier",
            "sid",
            "site_name",
            "assignment",
            "allocated",
            "allocated_user",
            "allocated_datetime",
            "allocated_site",
            "randomizer_name",
            *audit_fields,
        )
        return tuple(set(readonly_fields))

    def get_queryset(self, request):
        """
        Filter the changelist to show for this site_name only.
        """
        site = Site.objects.get(pk=settings.SITE_ID)
        qs = self.model.objects.filter(site_name=site.name)
        ordering = self.get_ordering(request)
        if ordering:
            qs = qs.order_by(*ordering)
        return qs

    def get_list_display(self, request) -> tuple[str, ...]:
        fields = (
            "sid",
            "assignment",
            "site_name",
            "subject_identifier",
            "allocated_datetime",
            "allocated_site",
            "randomizer_name",
        )
        if flds := site_randomizers.get_by_model(
            self.model._meta.label_lower
        ).get_extra_list_display():
            fields = list(fields)
            for pos, fname in flds:
                fields.insert(pos, fname)
            fields = tuple(fields)
        if user_is_blinded(request.user.username) or (
            not user_is_blinded(request.user.username)
            and RANDO_UNBLINDED not in [g.name for g in request.user.groups.all()]
        ):
            fields = tuple([fname for fname in fields if fname != "assignment"])
        return fields

    @staticmethod
    def get_fieldnames(request) -> tuple[str, ...]:
        fields = (
            "subject_identifier",
            "sid",
            "assignment",
            "allocated",
            "allocated_user",
            "allocated_datetime",
            "allocated_site",
            "randomizer_name",
        )
        if user_is_blinded(request.user.username) or (
            not user_is_blinded(request.user.username)
            and RANDO_UNBLINDED not in [g.name for g in request.user.groups.all()]
        ):
            fields = tuple([fname for fname in fields if fname != "assignment"])
        return fields

    def get_list_filter(self, request) -> tuple[str, ...]:
        fields = (
            "assignment",
            "allocated_datetime",
            "allocated_site",
            "site_name",
            "randomizer_name",
        )
        if flds := site_randomizers.get_by_model(
            self.model._meta.label_lower
        ).get_extra_list_filter():
            fields = list(fields)
            for pos, fname in flds:
                fields.insert(pos, fname)
            fields = tuple(fields)
        if user_is_blinded(request.user.username) or (
            not user_is_blinded(request.user.username)
            and RANDO_UNBLINDED not in [g.name for g in request.user.groups.all()]
        ):
            fields = tuple([fname for fname in fields if fname != "assignment"])
        return fields


def register_admin():
    site_randomizers.autodiscover()
    for randomizer_cls in site_randomizers._registry.values():
        model = randomizer_cls.model_cls()
        admin_cls = type(f"{model.__name__}ModelAdmin", (RandomizationListModelAdmin,), {})
        with contextlib.suppress(AlreadyRegistered):
            edc_randomization_admin.register(model, admin_cls)


register_admin()



from django.db.models import CharField
from django.utils.translation import gettext as _

from edc_constants.choices import IDENTITY_TYPE


class IdentityTypeField(CharField):
    """
    have IdentityTypeField immediately follow an identity field:

    For example,

    ...

    identity = models.CharField(
        verbose_name=_("Identity number (OMANG, etc)"),
        max_length=25,
        unique=True,
        help_text=_(
            "Use Omang, Passport number, driver's license "
            "number or Omang receipt number")
        )

    identity_type = IdentityTypeField()

    ...

    Use the value of identity_type to check the cleaned
    value of identity at the form level.
    """

    description = _("Custom field for Identity Type")

    def __init__(self, *args, **kwargs):
        kwargs.setdefault("verbose_name", _("What type of identity number is this?"))
        kwargs.setdefault("editable", True)
        kwargs.setdefault("max_length", 15)
        kwargs.setdefault("choices", IDENTITY_TYPE)
        CharField.__init__(self, *args, **kwargs)



from django.contrib import admin

from ..admin_site import edc_export_admin
from ..models import UploadExportReceiptFile


@admin.register(UploadExportReceiptFile, site=edc_export_admin)
class UploadExportReceiptFileAdmin(admin.ModelAdmin):
    date_hierarchy = "created"

    list_display = (
        "file_name",
        "accepted",
        "duplicate",
        "total",
        "errors",
        "created",
        "user_created",
        "hostname_created",
    )

    list_filter = ("created", "hostname_created")



import csv
from datetime import datetime

from django.contrib.auth.models import User


def export_users(path):
    path = path or f"edc_users_{datetime.now().strftime('%Y%m%d%H%M%S')}.csv"
    user = {
        "username": None,
        "password": None,
        "is_staff": None,
        "is_active": None,
        "first_name": None,
        "last_name": None,
        "job_title": None,
        "email": None,
        "mobile": None,
        "alternate_email": None,
        "site_names": None,
        "role_names": None,
    }

    with open(path, "w+") as f:
        writer = csv.DictWriter(f, fieldnames=user, delimiter="|")
        writer.writeheader()
        for user in User.objects.all().order_by("username"):
            site_names = ",".join([s.name for s in user.userprofile.sites.all()])
            role_names = ",".join([g.name for g in user.userprofile.roles.all()])
            user = {
                "username": user.username,
                "password": user.password,
                "is_staff": user.is_staff,
                "is_active": user.is_active,
                "first_name": user.first_name,
                "last_name": user.last_name,
                "job_title": user.userprofile.job_title,
                "email": user.email,
                "mobile": user.userprofile.mobile,
                "alternate_email": user.userprofile.alternate_email,
                "site_names": site_names or "",
                "role_names": role_names or "",
            }
            writer.writerow(user)
    print(f"Done. See file `{path}` in the current directory.")



# Generated by Django 3.2.13 on 2022-09-18 02:08

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("edc_offstudy", "0013_alter_historicalsubjectoffstudy_options_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="historicalsubjectoffstudy",
            name="consent_model",
            field=models.CharField(editable=False, max_length=50, null=True),
        ),
        migrations.AddField(
            model_name="historicalsubjectoffstudy",
            name="consent_version",
            field=models.CharField(editable=False, max_length=10, null=True),
        ),
        migrations.AddField(
            model_name="subjectoffstudy",
            name="consent_model",
            field=models.CharField(editable=False, max_length=50, null=True),
        ),
        migrations.AddField(
            model_name="subjectoffstudy",
            name="consent_version",
            field=models.CharField(editable=False, max_length=10, null=True),
        ),
    ]



from django import forms
from django.core.exceptions import ObjectDoesNotExist

from edc_adverse_event.utils import get_ae_model


class SubjectTransferModelFormMixin:
    # verify transfer date is not on or after death
    def validate_death_date(self):
        if (
            self.cleaned_data.get("transfer_date")
            and getattr(self.death_report, "death_date", None)
            and self.cleaned_data.get("transfer_date") >= self.death_report.death_date
        ):
            raise forms.ValidationError("Invalid date. Cannot be on or after death_date")

    def death_report(self):
        try:
            return get_ae_model("death_report").objects.get(
                subject_identifier=self.cleaned_data.get("subject_identifier")
            )
        except ObjectDoesNotExist:
            return None



from .aliquot import (
    AliquotIdentifierModelMixin,
    AliquotLabelMixin,
    AliquotModelMixin,
    AliquotShippingMixin,
    AliquotTypeModelMixin,
)
from .panel_model_mixin import LabProfileError, PanelModelError, PanelModelMixin
from .requisition import (
    CrfWithRequisitionModelMixin,
    RequisitionIdentifierMixin,
    RequisitionModelMixin,
    RequisitionStatusMixin,
    requisition_fk_options,
)
from .result import ResultItemModelMixin, ResultModelMixin
from .shipping import ManifestModelMixin, VerifyBoxModelMixin, VerifyModelMixin



import socket

from django.db.models import CharField
from django.utils.translation import gettext as _


class HostnameModificationField(CharField):
    description = _("Custom field for hostname modified")

    def __init__(self, *args, **kwargs):
        kwargs.setdefault("blank", True)
        CharField.__init__(self, *args, **kwargs)

    def pre_save(self, model_instance, add):
        """Updates socket.gethostname() on each save."""
        value = socket.gethostname()
        setattr(model_instance, self.attname, value)
        return value



from .fasting_model_mixin import FastingModelMixin
from .fbg_model_mixin import FbgModelMixin
from .glucose_model_mixin import GlucoseModelMixin
from .hba1c_model_mixin import Hba1cModelMixin
from .ogtt_model_mixin import OgttModelMixin



# Generated by Django 2.1 on 2018-10-06 23:03

from django.db import migrations

from edc_action_item.data_fixers import fix_null_historical_action_identifier


def fix(apps, schema_editor):
    fix_null_historical_action_identifier(app_label="edc_locator", models=["subjectlocator"])


class Migration(migrations.Migration):
    dependencies = [("edc_locator", "0012_auto_20181010_2121")]

    operations = [migrations.RunPython(fix)]



# Generated by Django 4.0.7 on 2022-08-23 13:24

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("edc_locator", "0025_auto_20220704_1841"),
    ]

    operations = [
        migrations.AlterField(
            model_name="historicalsubjectlocator",
            name="action_identifier",
            field=models.CharField(blank=True, db_index=True, max_length=50),
        ),
        migrations.AlterField(
            model_name="subjectlocator",
            name="action_identifier",
            field=models.CharField(blank=True, max_length=50, unique=True),
        ),
    ]
